Package: mitch
Title: Multi-Contrast Gene Set Enrichment Analysis
Version: 1.24.0
Authors@R: c(
    person(given = "Mark",
        family = "Ziemann",
        role = c("aut","cre","cph"),
        email = "mark.ziemann@gmail.com",
        comment = c(ORCID="0000-0002-7688-6974")),
    person(given="Antony",
        family="Kaspi",
        role = c("aut","cph"))
    )
Description: mitch is an R package for multi-contrast enrichment analysis. At
    it’s heart, it uses a rank-MANOVA based statistical approach to detect sets
    of genes that exhibit enrichment in the multidimensional space as compared
    to the background. The rank-MANOVA concept dates to work by Cox and Mann 
    (https://doi.org/10.1186/1471-2105-13-S16-S12). mitch is useful for 
    pathway analysis of profiling studies with one, two or more contrasts,
    or in studies with multiple omics profiling, for example proteomic,
    transcriptomic, epigenomic analysis of the same samples. mitch is perfectly
    suited for pathway level differential analysis of scRNA-seq data.
    We have an established routine for pathway enrichment of Infinium Methylation
    Array data (see vignette).
    The main strengths of mitch are that it can import datasets easily from
    many upstream tools and has advanced plotting features to visualise these
    enrichments. 
Depends: R (>= 4.4)
Suggests: stringi, testthat (>= 2.1.0), HGNChelper,
        IlluminaHumanMethylation450kanno.ilmn12.hg19,
        IlluminaHumanMethylationEPICanno.ilm10b4.hg19
Imports: stats, grDevices, graphics, utils, MASS, plyr, reshape2,
        parallel, GGally, grid, gridExtra, knitr, rmarkdown, ggplot2,
        gplots, beeswarm, echarts4r, kableExtra, dplyr, network
License: CC BY-SA 4.0 + file LICENSE
Encoding: UTF-8
URL: https://github.com/markziemann/mitch
BugReports: https://github.com/markziemann/mitch
LazyData: false
RoxygenNote: 7.3.2
biocViews: GeneExpression, GeneSetEnrichment, SingleCell,
        Transcriptomics, Epigenetics, Proteomics,
        DifferentialExpression, Reactome, DNAMethylation,
        MethylationArray, DataImport
VignetteBuilder: knitr
git_url: https://git.bioconductor.org/packages/mitch
git_branch: RELEASE_3_23
git_last_commit: 1235d24
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.23
Date/Publication: 2026-04-28
NeedsCompilation: no
Packaged: 2026-04-28 22:53:24 UTC; biocbuild
Author: Mark Ziemann [aut, cre, cph] (ORCID:
    <https://orcid.org/0000-0002-7688-6974>),
  Antony Kaspi [aut, cph]
Maintainer: Mark Ziemann <mark.ziemann@gmail.com>
Built: R 4.6.0; ; 2026-04-29 04:50:55 UTC; unix

# Generated by roxygen2: do not edit by hand

export(gmt_import)
export(mitch_calc)
export(mitch_import)
export(mitch_plots)
export(mitch_report)
export(network_genes)
export(networkplot)
import(GGally)
import(echarts4r)
import(ggplot2)
import(grDevices)
import(graphics)
import(grid)
import(gridExtra)
import(kableExtra)
import(knitr)
import(parallel)
import(stats)
import(utils)
importFrom(MASS,kde2d)
importFrom(beeswarm,beeswarm)
importFrom(dplyr,bind_rows)
importFrom(dplyr,mutate)
importFrom(gplots,heatmap.2)
importFrom(network,network)
importFrom(plyr,join_all)
importFrom(plyr,ldply)
importFrom(reshape2,melt)
importFrom(rmarkdown,render)

# mitch

mitch is an R package for multi-dimensional enrichment analysis. At it's heart, it uses a rank-MANOVA based statistical approach to detect sets of genes that exhibit enrichment in the multidimensional space as compared to the background. Mitch is useful for pathway analysis of profiling studies with two to or more contrasts, or in studies with multiple omics profiling, for example proteomic, transcriptomic, epigenomic analysis of the same samples. Mitch is perfectly suited for pathway level differential analysis of scRNA-seq data.

<img align="center" width="160" height="200" src="https://github.com/markziemann/mitch_paper/blob/master/figs/mitch.png">

## Installation

```
if(!requireNamespace("BiocManager", quietly = TRUE))
    install.packages("BiocManager")
BiocManager::install("mitch")

library("mitch")
```

## Workflow overview

### Importing gene sets

mitch has a function to import GMT files to R lists (adapted from [Yu et al, 2012](https://dx.doi.org/10.1089%2Fomi.2011.0118) in the [clusterProfiler](http://bioconductor.org/packages/release/bioc/html/clusterProfiler.html) package). For example:

```
download.file("https://reactome.org/download/current/ReactomePathways.gmt.zip", destfile="ReactomePathways.gmt.zip")
unzip("ReactomePathways.gmt.zip")
genesets <- gmt_import("ReactomePathways.gmt")
```

### Importing profiling data

mitch accepts pre-ranked data supplied by the user, but also has a function called `mitch_import` for importing tables generated by Limma, edgeR, DESeq2, ABSSeq, Sleuth, Seurat and Muscat. By default, only the genes that are detected in all contrasts are included, but this behaviour can be modified. The below example imports two edgeR tables called "dge1" and "dge2". Where gene identifiers are coded in the row names.

```
x <- list("dge1"=dge1, "dge2"=dge2)
y <- mitch_import(x, DEtype="edger")
```

mitch can do unidimensional analysis if you provide it a single profile as a dataframe. 

```
y <- mitch_import(df, DEtype="edger")
```

If the gene identifiers are not given in the rownames, then the column can be specified with the `geneIDcol` parameter like this:

```
y <- mitch_import(df, DEtype="edger", geneIDcol="MyGeneIDs")
```

For edgeR, differential gene expression is scored using the directional nominal p-value.

S = -log10(p-value) * sgn(logFC)

For DESeq2, the test statistic is used.
If this is not desired, then users can perform their own custom scoring procedure.

There are many cases where the gene IDs don't match the gene sets. To overcome this, `mitch_import` also accepts a two-column table that relates gene identifiers in the profiling data to those in the gene sets. 

`?mitch_import` provides more instructions on using this feature.

### Calculating enrichment

The `mitch_calc` function performs multivariate enrichment analysis of the supplied gene sets in the scored profiling data.  At its simpest form `mitch_calc` function accepts the scored data as the first argument and the genesets as the second argument. Users can prioritise enrichments based on small adjusted p-values, or by the observed effect size (magnitude of the enrichment score).

In general, effect size prioritisation will give you more specific and actionable results, that
highlights relatively small gene sets with large magnitude of change.
If you prioritise by significance, you will highlight larger gene sets with more subtle expression
changes.
As a general rule, our group typically presents enrichment results prioritised by effect size after
excluding gene sets with FDR>0.05.

```
res <- mitch_calc(y, genesets, priority="significance")
res <- mitch_calc(y, genesets, priority="effect")
```

In some cases, you may want to prioritise the results based on the diversity of gene expression
differences across multiple contrasts.
This will highlight gene sets that have opposite and diverse s.dist scores across contrasts.
This is particularly interesting for scRNA-seq analysis where you may want to know how different cell
types respond to a stimulus, so you are interested in the gene sets with the biggest variation in
s.dist scores across the contrasts.

```
res <- mitch_calc(y, genesets, priority="SD")
```

Note that SD prioritisation won't work for 1D analysis.

One `mitch_calc()` is finished, you can peek at the top results with `head` like this:

```
head(res$enrichment_result)
```

By default, `mitch_calc` uses only thread for calculation, but you can speed it up using more
parallel threads, by setting the `cores` option.
We suggest 8-12 parallel threads for returning results the fastest.
Using more than 12 doesn't yield faster results typically.
Keep in mind that the more parallel threads used, the more RAM is required.

```
res <- mitch_calc(y, genesets, priority="significance", cores=4)
```

By default, gene sets with fewer than 10 members present in the profiling data are discarded. This threshold can be modified using the `minsetsize` option. There is no upper limit of gene set size.

```
res <- mitch_calc(y, genesets, priority="significance", minsetsize=5)
```

By default, in downstream visualisation steps, charts are made from the top 50 gene sets, but this can be modified using the `resrows` option. 

```
res <- mitch_calc(y, genesets, priority="significance", resrows=100)
```

### Generate a HTML report

Can be done like this:

```
mitch_report(res, "myreport.html")
```

Take a look at an [example report](https://github.com/markziemann/mitch_paper/blob/master/figs/myreport.html).

### Generate high resolution plots

In case you want the charts in PDF format, these can be generated as such:

```
mitch_plots(res, outfile="mycharts.pdf")
```

Take a look at an [example plot set](https://github.com/markziemann/mitch_paper/blob/master/figs/mycharts.pdf).


### Network plot

For one-dimensional analysis, you can generate a network plot to visualise the similarity of the driver gene members for the top (20) up- and downregulated sets meeting a significance cut-off (FDR<0.05).
These cut-offs can be altrered.

```
networkplot(res,FDR=0.05,n_sets=20)
```

See `?networkplot()` for more details.

If you want to see which genes are contributing to the network chart, use the `network_genes()` function.

```
network_genes(FDR=0.05,n_sets=20)
```

### scRNA-seq

This type of data is notoriously sparse, so pseudobulk aggregation is recommended.
Mitch works best if there are >1000 genes detected in each cell type (contrast).
If there are <400 genes present it may cause mitch to run into an error in the MANOVA test due to insufficient degrees of freedom.
It is recommended to run the analysis with all three prioritisation schemes (significance, effect and
SD) to interpret the data.

### Infinium DNA methylation data

Mitch can be applied to Infinium Beadarray methylation data if you have limma results for probes.
Please see our solution workflow on [Bioconductor](https://www.bioconductor.org/packages/devel/bioc/vignettes/mitch/inst/doc/infiniumMethArrayWorkflow.html).

### Contrbutions

If you have questions or need help with applying mitch to your work, please raise an issue.

In the inst/extdata/ folder there are some "raw" data files which are used to
demonstrate mitch functions in the examples. The instructions used to create
these data are in the inst/script/example_datagen.R script.

These are the files:
.
├── k36a.tsv 
├── k9a.tsv
├── rna.tsv
└── sample_genesets.gmt

These three tables are edgeR differential expression/binding tables comparing
samples before and after 60 minutes stimulation. The 'rna.tsv' file is for 
gene expression measured by RNA-seq. The 'k9a.tsv' file represents histone 3 
lysine 9 acetylation responses measured with ChIP-seq. The 'k36a.tsv' file
represents histone 3 lysine 36 acetylation responses measured with ChIP-seq. 
These files represent only 1000 genes to keep file sizes small for the 
purposes of the examples. The full files are to be found in another repo:
https://github.com/markziemann/mitch_paper/tree/master/data-raw

## ----install, eval = FALSE----------------------------------------------------
# if(!requireNamespace("BiocManager", quietly = TRUE))
#     install.packages("BiocManager")
# BiocManager::install("mitch")

## ----lib----------------------------------------------------------------------

suppressPackageStartupMessages({
    library("mitch")
    library("dplyr")
    library("HGNChelper")
    library("IlluminaHumanMethylation450kanno.ilmn12.hg19")
    library("IlluminaHumanMethylationEPICanno.ilm10b4.hg19")
})

## ----genesetsExample----------------------------------------------------------
data(genesetsExample)
head(genesetsExample,3)

## ----anno1--------------------------------------------------------------------
anno <- getAnnotation(IlluminaHumanMethylation450kanno.ilmn12.hg19)
myann <- data.frame(anno[,c("UCSC_RefGene_Name","UCSC_RefGene_Group","Islands_Name","Relation_to_Island")])
head(myann)
gp <- myann[,"UCSC_RefGene_Name",drop=FALSE]
gp2 <- strsplit(gp$UCSC_RefGene_Name,";")
names(gp2) <- rownames(gp)
gp2 <- lapply(gp2,unique)
gt1 <- stack(gp2)
colnames(gt1) <- c("gene","probe")
gt1$probe <- as.character(gt1$probe)
dim(gt1)
head(gt1)

## ----genenames----------------------------------------------------------------
#new.hgnc.table <- getCurrentHumanMap()
new.hgnc.table <- readRDS("../inst/extdata/new.hgnc.table.rds")
fix <- checkGeneSymbols(gt1$gene,map=new.hgnc.table)
fix2 <- fix[which(fix$x != fix$Suggested.Symbol),]
length(unique(fix2$x))
gt1$gene <- fix$Suggested.Symbol
head(gt1)

## ----import1------------------------------------------------------------------
x <- read.table("https://ziemann-lab.net/public/gmea/dma3a.tsv",header=TRUE,row.names=1)
head(x)

## ----import2------------------------------------------------------------------
y <- mitch_import(x,DEtype="limma",geneTable=gt1)
head(y)
dim(y)

## ----enrich1------------------------------------------------------------------
res<-mitch_calc(y,genesetsExample,priority="effect",cores=2,minsetsize=5)
head(res$enrichment_result,10)

## ----downstream1--------------------------------------------------------------

mitch_plots(res,outfile="methcharts.pdf")


## ----downstream2,results=FALSE------------------------------------------------

mitch_report(res,"methreport.html")


## ----meth_network,fig.width=14,fig.height=6-----------------------------------

networkplot(res,FDR=0.1,n_sets=10)

network_genes(res,FDR=1,n_sets=10)


## ----EPIC---------------------------------------------------------------------
anno <- getAnnotation(IlluminaHumanMethylationEPICanno.ilm10b4.hg19)
myann <- data.frame(anno[,c("UCSC_RefGene_Name","UCSC_RefGene_Group","Islands_Name","Relation_to_Island")])
gp <- myann[,"UCSC_RefGene_Name",drop=FALSE]
gp2 <- strsplit(gp$UCSC_RefGene_Name,";")
names(gp2) <- rownames(gp)
gp2 <- lapply(gp2,unique)
gt <- stack(gp2)
colnames(gt) <- c("gene","probe")
gt$probe <- as.character(gt$probe)
dim(gt)
str(gt)

## ----sessioninfo,message=FALSE------------------------------------------------
sessionInfo()

#install.packages("../..",repos = NULL, type="source")
library(mitch)

DATAURL=paste("https://raw.githubusercontent.com/",
"markziemann/mitch_paper/master/data-raw/",sep="")
RNAURL=paste(DATAURL,"rna.tsv",sep="")
rna <- read.table(RNAURL, header=TRUE)
K9AURL=paste(DATAURL,"k9a.tsv",sep="")
k9a <- read.table(K9AURL, header = TRUE)
K36AURL=paste(DATAURL,"k36a.tsv",sep="")
k36a <- read.table(K36AURL, header = TRUE)
myList <- list(rna = rna, k9a = k9a, k36a = k36a)

myImportedData <- mitch_import(myList, "edger")
myImportedData <- head(myImportedData, 1000)

rna <- rna[rownames(rna) %in% rownames(myImportedData), ]
k9a <- k9a[rownames(k9a) %in% rownames(myImportedData), ]
k36a <- k36a[rownames(k36a) %in% rownames(myImportedData), ]

myList <- list(rna = rna, k9a = k9a, k36a = k36a)

REACTOMEURL=paste(DATAURL,"ReactomePathways.gmt",sep="")
download.file(REACTOMEURL,destfile="ReactomePathways.gmt")
genesetsExample <- head(gmt_import(REACTOMEURL), 200)
reactome<-readLines("ReactomePathways.gmt")
reactome<-head(reactome,200)
writeLines(reactome,"../../inst/extdata/sample_genesets.gmt")
	
resExample<-mitch_calc(myImportedData,genesetsExample,
resrows=5,priority="significance")

save(genesetsExample,file="../../data/genesetsExample.RData",compress="xz")
save(k36a, file = "../../data/k36a.RData", compress = "xz")
save(k9a, file = "../../data/k9a.RData", compress = "xz")
save(rna, file = "../../data/rna.RData", compress = "xz")
save(myList, file = "../../data/myList.RData", compress = "xz")
save(myImportedData, file = "../../data/myImportedData.RData",compress="xz")
save(resExample, file = "../../data/resExample.RData", compress = "xz")

write.table(rna, file = "../../inst/extdata/rna.tsv",
sep = "\t", quote = FALSE)
write.table(k9a, file = "../../inst/extdata/k9a.tsv",
sep = "\t", quote = FALSE)
write.table(k36a, file = "../../inst/extdata/k36a.tsv",
sep = "\t", quote = FALSE)

message("1d")
myImportedData <- mitch_import(rna, "edger" )
resExample <- mitch_calc(myImportedData, genesetsExample,
resrows = 5, priority = "significance")
nrow(resExample$enrichment_result)

message("2d")
myList <- list(rna = rna, k9a = k9a)
myImportedData <- mitch_import(myList, "edger" )
resExample <- mitch_calc(myImportedData, genesetsExample,
resrows = 5, priority = "significance")
nrow(resExample$enrichment_result)

message("3d")
myList <- list(rna = rna, k9a = k9a, k36a = k36a)
myImportedData <- mitch_import(myList, "edger" )
resExample <- mitch_calc(myImportedData, genesetsExample,
resrows = 5, priority = "significance")
nrow(resExample$enrichment_result)

unlink("ReactomePathways.gmt")

## ----install, eval = FALSE----------------------------------------------------
# if(!requireNamespace("BiocManager", quietly = TRUE))
#     install.packages("BiocManager")
# BiocManager::install("mitch")

## ----lib----------------------------------------------------------------------
library("mitch")

## ----gsets--------------------------------------------------------------------
download.file("https://reactome.org/download/current/ReactomePathways.gmt.zip",
    destfile="ReactomePathways.gmt.zip")
unzip("ReactomePathways.gmt.zip")
genesets <- gmt_import("ReactomePathways.gmt")

## ----genesetsExample----------------------------------------------------------
data(genesetsExample)
head(genesetsExample,3)

## ----import11-----------------------------------------------------------------
data(rna,k9a)
x <- list("rna"=rna,"k9a"=k9a)
y <- mitch_import(x,"edgeR")
head(y)

## ----import4------------------------------------------------------------------
y <- mitch_import(rna,DEtype="edger")
head(y)

## ----import5------------------------------------------------------------------
# first rearrange cols
rna_mod <- rna
rna_mod$MyGeneIDs <- rownames(rna_mod)
rownames(rna_mod) <- seq(nrow(rna_mod))
head(rna_mod)
# now import with geneIDcol
y <- mitch_import(rna_mod,DEtype="edgeR",geneIDcol="MyGeneIDs")
head(y)

## ----import6------------------------------------------------------------------
library("stringi")
# obtain vector of gene names
genenames <- rownames(rna)
# create fake accession numbers
accessions <- paste("Gene0",stri_rand_strings(nrow(rna)*2, 6, pattern = "[0-9]"),sep="")
accessions <- head(unique(accessions),nrow(rna))
# create a gene table file that relates gene names to accession numbers
gt <- data.frame(genenames,accessions)

# now swap gene names for accessions
rna2 <- merge(rna,gt,by.x=0,by.y="genenames")
rownames(rna2) <- rna2$accessions
rna2 <- rna2[,2:5]

k9a2 <- merge(k9a,gt,by.x=0,by.y="genenames")
rownames(k9a2) <- k9a2$accessions
k9a2 <- k9a2[,2:5]

# now have a peek at the input data before importing
head(rna2,3)
head(k9a2,3)
head(gt,3)
x <- list("rna2"=rna2,"k9a2"=k9a2)
y <- mitch_import(x,DEtype="edgeR",geneTable=gt)
head(y,3)

## ----calc1,results="hide"-----------------------------------------------------
# prioritisation by significance
res <- mitch_calc(y,genesetsExample,priority="significance",cores=2)

## ----calc2--------------------------------------------------------------------
# peek at the results
head(res$enrichment_result)

## ----calc3,results="hide"-----------------------------------------------------
# prioritisation by effect size
res <- mitch_calc(y,genesetsExample,priority="effect",cores=2)

## ----calc4--------------------------------------------------------------------
head(res$enrichment_result)

## ----calc5,results="hide"-----------------------------------------------------
res <- mitch_calc(y,genesetsExample,priority="significance",minsetsize=5,cores=2)

## ----calc6,results="hide"-----------------------------------------------------
res<-mitch_calc(y,genesetsExample,priority="significance",resrows=3,cores=2)

## ----report,results="hide"----------------------------------------------------
mitch_report(res,"myreport.html",overwrite=TRUE)

## ----plots,results="hide"-----------------------------------------------------
mitch_plots(res,outfile="mycharts.pdf")

## ----network_demo,fig.width=14,fig.height=6-----------------------------------

networkplot(res,FDR=1,n_sets=10)

network_genes(res,FDR=1,n_sets=10)


## ----sessioninfo,message=FALSE------------------------------------------------
sessionInfo()

#' mitch: An R package for multi-dimensional pathway enrichment analysis
#'
#' mitch is an R package for multi-dimensional enrichment analysis. At it's 
#' heart, it uses a rank-MANOVA based statistical approach to detect sets of 
#' genes that exhibit enrichment in the multidimensional space as compared to 
#' the background. mitch is useful for pathway analysis of profiling studies 
#' with two to or more contrasts, or in studies with multiple omics profiling, 
#' for example proteomic, transcriptomic, epigenomic analysis of the same 
#' samples. mitch is perfectly suited for pathway level differential analysis 
#' of scRNA-seq data.
#'
#' A typical mitch workflow consists of:
#' 1) Import gene sets with gmt_import()
#' 2) Import profiling data with mitch_import()
#' 3) Calculate enrichments with mitch_calc()
#' 4) And generate plots and reports with mitch_plots() and mitch_report()
#'
#' More documentation on the github page https://github.com/markziemann/mitch
#' or with ?<function>, eg: ?mitch_import
#'
#' @docType package
#' @name mitch
#' @examples
#' # Example workflow
#' # Import some gene sets
#' genesetsExample<-gmt_import(system.file('extdata/sample_genesets.gmt', 
#' package = 'mitch'))
#' # Load some edgeR tables (rna, k9a, k36a). 
#' data(rna,k9a,k36a)
#' # Create a list of differential profiles
#' myList<-list('rna'=rna,'k9a'=k9a,'k36a'=k36a)
#' # Import as edgeR table 
#' myImportedData<-mitch_import(myList,DEtype='edger')
#' # Calculate enrichment using MANOVA
#' resExample<-mitch_calc(myImportedData,genesetsExample,priority='effect',
#' resrows=5,cores=2)
#' # Generate some high res plots in PDF format
#' mitch_plots(resExample,outfile='outres.pdf')
#' #' Generate a report of the analysis in HTML format
#' mitch_report(resExample,'outres.html')
"_PACKAGE"

#' @import utils
utils::globalVariables(c("p.adjustMANOVA", "effect", "p.adjustANOVA", "contrast",
    "value", "..density..","dummy_x","dummy_y"))


mapGeneIds <- function(y, z) {
    if (!is.null(attributes(y)$geneTable)) {
        gt <- attributes(y)$geneTable
        col1 <- length(which(z$geneidentifiers %in% gt[, 1]))
        col2 <- length(which(z$geneidentifiers %in% gt[, 2]))
        
        if (col1 + col2 < (nrow(y)/2)) {
            stop("Error it looks as if the Gene IDs in the profile don't match
            the geneTable")
        }
        
        if (col1 > col2) {
            colnames(gt) <- c("geneidentifiers", "GeneSymbol")
            z <- merge(gt, z, by = "geneidentifiers")
            z$geneidentifiers <- NULL
        } else {
            colnames(gt) <- c("GeneSymbol", "geneidentifiers")
            z <- merge(gt, z, by = "geneidentifiers")
            z$geneidentifiers <- NULL
        }
        z <- aggregate(. ~ GeneSymbol, z, function(x) {
            mean(as.numeric(as.character(x)))
        })

    }
    colnames(z) <- c("geneidentifiers", "y")
    z
}


edger_score <- function(y , geneIDcol = geneIDcol) {
    
    NCOL <- ncol(y)
    if (NCOL < 2) {
        stop("Error: there are <2 columns in the input, 'PValue' and 'logFC' 
        are required ")
    }
    
    PCOL <- length(which(names(y) == "PValue"))
    if (PCOL > 1) {
        stop("Error, there is more than 1 column named 'PValue' in the input")
    }
    if (PCOL < 1) {
        stop("Error, there is no column named 'PValue' in the input")
    }
    
    FCCOL <- length(which(names(y) == "logFC"))
    if (FCCOL > 1) {
        stop("Error, there is more than 1 column named 'logFC' in the input")
    }
    if (FCCOL < 1) {
        stop("Error, there is no column named 'logFC' in the input")
    }
    
    s <- sign(y$logFC) * -log10(y$PValue)
    
    if (!is.null(attributes(y)$geneIDcol)) {
        g <- y[, attributes(y)$geneIDcol]
    } else {
        g <- rownames(y)
    }
    z <- data.frame(g, s, stringsAsFactors = FALSE)
    colnames(z) <- c("geneidentifiers", "y")
    z <- mapGeneIds(y, z)
    z
}


deseq2_score <- function(y , geneIDcol = geneIDcol ) {

    ZCOL <- length(which(names(y) == "stat"))
    if (ZCOL > 1) {
        stop("Error, there is more than 1 column named 'stat' in the input")
    }
    if (ZCOL < 1) {
        stop("Error, there is no column named 'stat' in the input")
    }

    s <- y$stat

    if (!is.null(attributes(y)$geneIDcol)) {
        g <- y[, attributes(y)$geneIDcol]
    } else {
        g <- rownames(y)
    }
    z <- data.frame(g, s, stringsAsFactors = FALSE)
    colnames(z) <- c("geneidentifiers", "y")
    z <- mapGeneIds(y, z)
    z
}


absseq_score <- function(y, geneIDcol = geneIDcol ) {
    
    NCOL <- ncol(y)
    if (NCOL < 2) {
        stop("Error: there are <2 columns in the input, 'pvalue' and 
        'foldChange' are required ")
    }
    
    PCOL <- length(which(names(y) == "pvalue"))
    if (PCOL > 1) {
        stop("Error, there is more than 1 column named 'pvalue' in the input")
    }
    if (PCOL < 1) {
        stop("Error, there is no column named 'pvalue' in the input")
    }
    
    FCCOL <- length(which(names(y) == "foldChange"))
    if (FCCOL > 1) {
        stop("Error, there is more than 1 column named 'foldChange' in
        the input")
    }
    if (FCCOL < 1) {
        stop("Error, there is no column named 'foldChange' in the input")
    }
    
    s <- sign(y$foldChange) * -log10(y$pvalue)
    
    if (!is.null(attributes(y)$geneIDcol)) {
        g <- y[, attributes(y)$geneIDcol]
    } else {
        g <- rownames(y)
    }
    z <- data.frame(g, s, stringsAsFactors = FALSE)
    colnames(z) <- c("geneidentifiers", "y")
    z <- mapGeneIds(y, z)
    z
}


sleuth_score <- function(y , geneIDcol = geneIDcol ) {
    
    NCOL <- ncol(y)
    if (NCOL < 2) {
        stop("Error: there are <2 columns in the input, 'pval' and 'b'
        are required ")
    }
    
    PCOL <- length(which(names(y) == "pval"))
    if (PCOL > 1) {
        stop("Error, there is more than 1 column named 'pval' in the input")
    }
    if (PCOL < 1) {
        stop("Error, there is no column named 'pval' in the input")
    }
    
    FCCOL <- length(which(names(y) == "b"))
    if (FCCOL > 1) {
        stop("Error, there is more than 1 column named 'b' in the input")
    }
    if (FCCOL < 1) {
        stop("Error, there is no column named 'b' in the input")
    }
    
    s <- sign(y$b) * -log10(y$pval)
    
    if (!is.null(attributes(y)$geneIDcol)) {
        g <- y[, attributes(y)$geneIDcol]
    } else {
        g <- rownames(y)
    }
    z <- data.frame(g, s, stringsAsFactors = FALSE)
    colnames(z) <- c("geneidentifiers", "y")
    z <- mapGeneIds(y, z)
    z
}


topconfect_score <- function(y , geneIDcol = geneIDcol ) {
    
    FCCOL <- length(which(names(y) == "confect"))
    if (FCCOL > 1) {
        stop("Error, there is more than 1 column named 'confect' in the input")
    }
    if (FCCOL < 1) {
        stop("Error, there is no column named 'confect' in the input")
    }
    
    # better to get the sign of fold change from the effect column
    FCCOL <- length(which(names(y) == "effect"))
    if (FCCOL > 1) {
        stop("Error, there is more than 1 column named 'effect' in the input")
    }
    if (FCCOL < 1) {
        stop("Error, there is no column named 'effect' in the input")
    }
    
    # there is a problem with topconfects having some NA values
    yy <- y[!is.na(y$effect), ]
    
    pos <- subset(yy, effect > 0)
    neg <- subset(yy, effect < 0)
    pos$mitchrank <- rev(seq(from = 1, to = nrow(pos)))
    neg$mitchrank <- rev(seq(from = -1, to = -nrow(neg)))
    yy <- rbind(pos, neg)
    s <- yy$mitchrank
    
    if (!is.null(attributes(y)$geneIDcol)) {
        g <- yy[, attributes(y)$geneIDcol]
    } else {
        g <- rownames(yy)
    }
    z <- data.frame(g, s, stringsAsFactors = FALSE)
    colnames(z) <- c("geneidentifiers", "y")
    z <- mapGeneIds(y, z)
    z
}


ballgown_score <- function(y , geneIDcol = geneIDcol ) {

    NCOL <- ncol(y)
    if (NCOL < 2) {
        stop("Error: there are <2 columns in the input, 'pval' and 'fc' are
        required ")
    }

    PCOL <- length(which(names(y) == "pval"))
    if (PCOL > 1) {
        stop("Error, there is more than 1 column named 'pval' in the input")
    }
    if (PCOL < 1) {
        stop("Error, there is no column named 'pval' in the input")
    }

    FCCOL <- length(which(names(y) == "fc"))
    if (FCCOL > 1) {
        stop("Error, there is more than 1 column named 'fc' in the input")
    }
    if (FCCOL < 1) {
        stop("Error, there is no column named 'fc' in the input")
    }

    s <- sign(log2(y$fc)) * -log10(y$pval)

    if (!is.null(attributes(y)$geneIDcol)) {
        g <- y[, attributes(y)$geneIDcol]
    } else {
        g <- rownames(y)
    }
    z <- data.frame(g, s, stringsAsFactors = FALSE)
    colnames(z) <- c("geneidentifiers", "y")
    z <- mapGeneIds(y, z)
    z
}


noiseq_score <- function(y , geneIDcol = geneIDcol ) {

    ZCOL <- length(which(names(y) == "ranking"))
    if (ZCOL > 1) {
        stop("Error, there is more than 1 column named 'ranking' in the input")
    }
    if (ZCOL < 1) {
        stop("Error, there is no column named 'ranking' in the input")
    }

    s <- y$ranking

    if (!is.null(attributes(y)$geneIDcol)) {
        g <- y[, attributes(y)$geneIDcol]
    } else {
        g <- rownames(y)
    }
    z <- data.frame(g, s, stringsAsFactors = FALSE)
    colnames(z) <- c("geneidentifiers", "y")
    z <- mapGeneIds(y, z)
    z
}


tcc_score <- function(y , geneIDcol = geneIDcol ) {

    NCOL <- ncol(y)
    if (NCOL < 2) {
        stop("Error: there are <2 columns in the input, 'p.value' and 'm.value'
        are required ")
    }

    PCOL <- length(which(names(y) == "p.value"))
    if (PCOL > 1) {
        stop("Error, there is more than 1 column named 'p.value' in the input")
    }
    if (PCOL < 1) {
        stop("Error, there is no column named 'p.value' in the input")
    }

    FCCOL <- length(which(names(y) == "m.value"))
    if (FCCOL > 1) {
        stop("Error, there is more than 1 column named 'm.value' in the input")
    }
    if (FCCOL < 1) {
        stop("Error, there is no column named 'm.value' in the input")
    }

    s <- sign(y$m.value) * -log10(y$p.value)

    if (!is.null(attributes(y)$geneIDcol)) {
        g <- y[, attributes(y)$geneIDcol]
    } else {
        g <- rownames(y)
    }
    z <- data.frame(g, s, stringsAsFactors = FALSE)
    colnames(z) <- c("geneidentifiers", "y")
    z <- mapGeneIds(y, z)
    z
}


deds_score <- function(y , geneIDcol = geneIDcol ) {

    ZCOL <- length(which(names(y) == "t"))
    if (ZCOL > 1) {
        stop("Error, there is more than 1 column named 't' in the input")
    }
    if (ZCOL < 1) {
        stop("Error, there is no column named 't' in the input")
    }

    s <- y$t

    if (!is.null(attributes(y)$geneIDcol)) {
        g <- y[, attributes(y)$geneIDcol]
    } else {
        g <- rownames(y)
    }
    z <- data.frame(g, s, stringsAsFactors = FALSE)
    colnames(z) <- c("geneidentifiers", "y")
    z <- mapGeneIds(y, z)
    z
}


cuffdiff_score <- function(y , geneIDcol = geneIDcol ) {

    ZCOL <- length(which(names(y) == "test_stat"))
    if (ZCOL > 1) {
        stop("Error, there is more than 1 column named 'test_stat' in the
        input")
    }
    if (ZCOL < 1) {
        stop("Error, there is no column named 'test_stat' in the input")
    }

    s <- y$test_stat

    if (!is.null(attributes(y)$geneIDcol)) {
        g <- y[, attributes(y)$geneIDcol]
    } else {
        g <- rownames(y)
    }
    z <- data.frame(g, s, stringsAsFactors = FALSE)
    colnames(z) <- c("geneidentifiers", "y")
    z <- mapGeneIds(y, z)
    z
}


seurat_score <- function(y , geneIDcol = geneIDcol ) {

    colnames(y) <- gsub("avg_log2FC","avg_logFC",colnames(y))
    NCOL <- ncol(y)
    if (NCOL < 2) {
        stop("Error: there are <2 columns in the input, 'p_val' and 'avg_logFC'
        are required ")
    }
    
    PCOL <- length(which(names(y) == "p_val"))
    if (PCOL > 1) {
        stop("Error, there is more than 1 column named 'p_val' in the input")
    }
    if (PCOL < 1) {
        stop("Error, there is no column named 'p_val' in the input")
    }
    
    FCCOL <- length(which(names(y) == "avg_logFC"))
    if (FCCOL > 1) {
        stop("Error, there is more than 1 column named 'avg_logFC' in the
        input")
    }
    if (FCCOL < 1) {
        stop("Error, there is no column named 'avg_logFC' in the input")
    }
    
    s <- sign(y$avg_logFC) * -log10(y$p_val)
    
    if (!is.null(attributes(y)$geneIDcol)) {
        g <- y[, attributes(y)$geneIDcol]
    } else {
        g <- rownames(y)
    }
    z <- data.frame(g, s, stringsAsFactors = FALSE)
    colnames(z) <- c("geneidentifiers", "y")
    z <- mapGeneIds(y, z)
    
    z$y[is.infinite(z$y) & z$y < 0] <- min(z$y[!is.infinite(z$y)]) - 0.01
    z$y[is.infinite(z$y) & z$y > 0] <- max(z$y[!is.infinite(z$y)]) + 0.01
    z
}


muscat_score <- function(y , geneIDcol = geneIDcol ) {
    
    NCOL <- ncol(y)
    if (NCOL < 2) {
        stop("Error: there are <2 columns in the input, 'p_val' and 'logFC'
        are required ")
    }
    
    PCOL <- length(which(names(y) == "p_val"))
    if (PCOL > 1) {
        stop("Error, there is more than 1 column named 'p_val' in the input")
    }
    if (PCOL < 1) {
        stop("Error, there is no column named 'p_val' in the input")
    }
    
    FCCOL <- length(which(names(y) == "logFC"))
    if (FCCOL > 1) {
        stop("Error, there is more than 1 column named 'logFC' in the input")
    }
    if (FCCOL < 1) {
        stop("Error, there is no column named 'logFC' in the input")
    }
    
    s <- sign(y$logFC) * -log10(y$p_val)
    
    if (!is.null(attributes(y)$geneIDcol)) {
        g <- y[, attributes(y)$geneIDcol]
    } else {
        g <- rownames(y)
    }
    z <- data.frame(g, s, stringsAsFactors = FALSE)
    colnames(z) <- c("geneidentifiers", "y")
    z <- mapGeneIds(y, z)
    z
}

scde_score <- function(y , geneIDcol = geneIDcol ) {

    ZCOL <- length(which(names(y) == "Z"))
    if (ZCOL > 1) {
        stop("Error, there is more than 1 column named 'Z' in the input")
    }
    if (ZCOL < 1) {
        stop("Error, there is no column named 'Z' in the input")
    }

    s <- y$Z

    if (!is.null(attributes(y)$geneIDcol)) {
        g <- y[, attributes(y)$geneIDcol]
    } else {
        g <- rownames(y)
    }
    z <- data.frame(g, s, stringsAsFactors = FALSE)
    colnames(z) <- c("geneidentifiers", "y")
    z <- mapGeneIds(y, z)
    z
}


mast_score <- function(y , geneIDcol = geneIDcol ) {

    NCOL <- ncol(y)
    if (NCOL < 2) {
        stop("Error: there are <2 columns in the input, 'Pr(>Chisq)' and 'coef'
        are required ")
    }

    PCOL <- length(which(names(y) == "Pr(>Chisq)"))
    if (PCOL > 1) {
        stop("Error, there is more than 1 column named 'Pr(>Chisq)' in the
        input")
    }
    if (PCOL < 1) {
        stop("Error, there is no column named 'Pr(>Chisq)' in the input")
    }

    FCCOL <- length(which(names(y) == "coef"))
    if (FCCOL > 1) {
        stop("Error, there is more than 1 column named 'coef' in the input")
    }
    if (FCCOL < 1) {
        stop("Error, there is no column named 'coef' in the input")
    }

    # because data.table object  doesn't seem to work
    y<-as.data.frame(y)

    s <- sign(y$coef) * -log10(y[,"Pr(>Chisq)"])

    if (!is.null(attributes(y)$geneIDcol)) {
        g <- y[, attributes(y)$geneIDcol]
    } else {
        g <- rownames(y)
    }
    z <- data.frame(g, s, stringsAsFactors = FALSE)
    colnames(z) <- c("geneidentifiers", "y")
    z <- mapGeneIds(y, z)
    z
}


desingle_score <- function(y , geneIDcol = geneIDcol ) {

    NCOL <- ncol(y)
    if (NCOL < 2) {
        stop("Error: there are <2 columns in the input, 'pvalue' and 
        'foldChange' are required ")
    }

    PCOL <- length(which(names(y) == "pvalue"))
    if (PCOL > 1) {
        stop("Error, there is more than 1 column named 'pvalue' in the input")
    }
    if (PCOL < 1) {
        stop("Error, there is no column named 'pvalue' in the input")
    }

    FCCOL <- length(which(names(y) == "foldChange"))
    if (FCCOL > 1) {
        stop("Error, there is more than 1 column named 'foldChange' in the
        input")
    }
    if (FCCOL < 1) {
        stop("Error, there is no column named 'foldChange' in the input")
    }

    s <- sign(log2(y$foldChange)) * -log10(y$pvalue)

    if (!is.null(attributes(y)$geneIDcol)) {
        g <- y[, attributes(y)$geneIDcol]
    } else {
        g <- rownames(y)
    }
    z <- data.frame(g, s, stringsAsFactors = FALSE)
    colnames(z) <- c("geneidentifiers", "y")
    z <- mapGeneIds(y, z)
    z
}


dmrcate_score <- function(y , geneIDcol = geneIDcol ) {

    NCOL <- ncol(y)
    if (NCOL < 2) {
        stop("Error: there are <2 columns in the input, 'Stouffer' and 
        'meanbetafc' are required ")
    }

    PCOL <- length(which(names(y) == "Stouffer"))
    if (PCOL > 1) {
        stop("Error, there is more than 1 column named 'Stouffer' in the
        input")
    }
    if (PCOL < 1) {
        stop("Error, there is no column named 'Stouffer' in the input")
    }

    FCCOL <- length(which(names(y) == "meanbetafc"))
    if (FCCOL > 1) {
        stop("Error, there is more than 1 column named 'meanbetafc' in the
        input")
    }
    if (FCCOL < 1) {
        stop("Error, there is no column named 'meanbetafc' in the input")
    }

    y<-as.data.frame(y)
    s <- sign(y$meanbetafc) * -log10(y$Stouffer)

    if (!is.null(attributes(y)$geneIDcol)) {
        g <- y[, attributes(y)$geneIDcol]
    } else {
        g <- rownames(y)
    }
    z <- data.frame(g, s, stringsAsFactors = FALSE)
    colnames(z) <- c("geneidentifiers", "y")
    z <- mapGeneIds(y, z)
    z
}


dep_score <- function(y , geneIDcol = geneIDcol ) {

    NCOL <- ncol(y)
    if (NCOL < 2) {
        stop("Error: there are <2 columns in the input, '*p.val' and '*ratio' 
        are required ")
    }

    PCOL <- length(grep("p.val",names(y)))
    if (PCOL > 1) {
        message("Note, using the leftmost column with '*p.val' in the name")
    }
    if (PCOL < 1) {
        stop("Error, there is no column named '*p.val' in the input")
    }

    FCCOL <- length(grep("ratio",names(y)))
    if (FCCOL > 1) {
        message("Note, using the leftmost column with 'ratio' in the name")
    }
    if (FCCOL < 1) {
        stop("Error, there is no column named 'ratio' in the input")
    }

    PCOL <- grep("p.val",names(y))[1]

    FCCOL <- grep("ratio",names(y))[1]

    s <- sign(y[,FCCOL]) * -log10(y[,PCOL])

    if (!is.null(attributes(y)$geneIDcol)) {
        g <- y[, attributes(y)$geneIDcol]
    } else {
        g <- rownames(y)
    }
    z <- data.frame(g, s, stringsAsFactors = FALSE)
    colnames(z) <- c("geneidentifiers", "y")
    z <- mapGeneIds(y, z)
    z
}


msmstests_score <- function(y , geneIDcol = geneIDcol ) {

    NCOL <- ncol(y)
    if (NCOL < 2) {
        stop("Error: there are <2 columns in the input, 'p.value' and 'LogFC'
        are required ")
    }

    PCOL <- length(which(names(y) == "p.value"))
    if (PCOL > 1) {
        stop("Error, there is more than 1 column named 'p.value' in the input")
    }
    if (PCOL < 1) {
        stop("Error, there is no column named 'p.value' in the input")
    }

    FCCOL <- length(which(names(y) == "LogFC"))
    if (FCCOL > 1) {
        stop("Error, there is more than 1 column named 'LogFC' in the input")
    }
    if (FCCOL < 1) {
        stop("Error, there is no column named 'LogFC' in the input")
    }

    s <- sign(y$LogFC) * -log10(y$p.value)

    if (!is.null(attributes(y)$geneIDcol)) {
        g <- y[, attributes(y)$geneIDcol]
    } else {
        g <- rownames(y)
    }
    z <- data.frame(g, s, stringsAsFactors = FALSE)
    colnames(z) <- c("geneidentifiers", "y")
    z <- mapGeneIds(y, z)
    z
}


plgem_score <- function(y , geneIDcol = geneIDcol ) {

    LEN <- length(y)
    if (LEN < 2) {
        stop("Error: there are <2 items in the input list, '$p.value' and
        '$PLGEM.STN' are required")
    }

    PCOL <- length(which(names(y) == "p.value"))
    if (PCOL > 1) {
        stop("Error, there is more than 1 item in the list named 'p.value'")
    }
    if (PCOL < 1) {
        stop("Error, there is no item in the list named 'p.value'")
    }

    FCCOL <- length(which(names(y) == "PLGEM.STN"))
    if (FCCOL > 1) {
        stop("Error, there is more than 1 column named 'PLGEM.STN' in the
        input")
    }
    if (FCCOL < 1) {
        stop("Error, there is no column named 'PLGEM.STN' in the input")
    }

    s <- sign(y$PLGEM.STN) * -log10(y$p.value)
    g <- rownames(y$PLGEM.STN)
    z <- data.frame(g, s, stringsAsFactors = FALSE)
    colnames(z) <- c("geneidentifiers", "y")
    z <- mapGeneIds(y, z)
    z
}


sdams_score <- function(y , geneIDcol = geneIDcol ) {

    LEN <- length(y)
    if (LEN < 2) {
        stop("Error: there are <2 items in the input list, '$pv_2part' and
        '$beta' are required")
    }

    PCOL <- length(which(names(y) == "pv_2part"))
    if (PCOL > 1) {
        stop("Error, there is more than 1 item in the list named 'pv_2part'")
    }
    if (PCOL < 1) {
        stop("Error, there is no item in the list named 'pv_2part'")
    }

    FCCOL <- length(which(names(y) == "beta"))
    if (FCCOL > 1) {
        stop("Error, there is more than 1 column named 'beta' in the input")
    }
    if (FCCOL < 1) {
        stop("Error, there is no column named 'beta' in the input")
    }

    s <- sign(y$beta) * -log10(y$pv_2part)
    g <- y$feat.names
    z <- data.frame(g, s, stringsAsFactors = FALSE)
    colnames(z) <- c("geneidentifiers", "y")
    z <- mapGeneIds(y, z)
    z
}


diffbind_score <- function(y , geneIDcol = geneIDcol ) {

    NCOL <- ncol(y)
    if (NCOL < 2) {
        stop("Error: there are <2 columns in the input, 'p.value' and 'Fold'
        are required ")
    }

    PCOL <- length(which(names(y) == "p.value"))
    if (PCOL > 1) {
        stop("Error, there is more than 1 column named 'p.value' in the input")
    }
    if (PCOL < 1) {
        stop("Error, there is no column named 'p.value' in the input")
    }

    FCCOL <- length(which(names(y) == "Fold"))
    if (FCCOL > 1) {
        stop("Error, there is more than 1 column named 'Fold' in the input")
    }
    if (FCCOL < 1) {
        stop("Error, there is no column named 'Fold' in the input")
    }

    y <- as.data.frame(y)
    s <- sign(y$Fold) * -log10(y$p.value)

    if (!is.null(attributes(y)$geneIDcol)) {
        g <- y[, attributes(y)$geneIDcol]
    } else {
        g <- rownames(y)
    }
    z <- data.frame(g, s, stringsAsFactors = FALSE)
    colnames(z) <- c("geneidentifiers", "y")
    z <- mapGeneIds(y, z)
    z
}


preranked_score <- function(y, joinType , geneIDcol ) {

    if (!is.null(attributes(y)$geneIDcol)) {
        NCOL <- ncol(y)
        if (NCOL > 2) {
            stop("Error: there are >2 columns in the input. Your files need to 
            have only the gene ID and rank stat ")
        }
        g <- y[, attributes(y)$geneIDcol]
        s <- y[ , !(names(y) %in% geneIDcol)]
    } else {
        NCOL <- ncol(y)
        if (NCOL > 1) {
            stop("Error: there are >1 columns in the input. Should only contain
            the gene ID and rank stat ")
        }
        g <- rownames(y)
        s <- y[,1]
    }
    z <- data.frame(g, s, stringsAsFactors = FALSE)
    colnames(z) <- c("geneidentifiers", "y")
    if ( is.null(joinType) ) {
        joinType <- "inner" 
    }
    if ( joinType == "inner" ) {
        z <- na.omit(z)
    }
    z <- mapGeneIds(y, z)
    z
}


#' mitch_import
#'
#' This function imports differential omics data from common differential tools
#' like edgeR, limma and DESeq2. It calculates a summarised differential
#' expression metric by multiplying the sign of the log fold change by the 
#' -log10 of the p-value. If this behaviour is not desired, mitch_import can be
#' bypassed in favour of another scoring metric.
#' @param x a list of differential expression tables.
#' @param DEtype the program that generated the differential expression table
#' Valid options are 'edgeR', 'DESeq2', 'limma', 'ABSSeq', 'Sleuth', 'Seurat',
#' 'topConfects', 'muscat', 'Swish', 'scDE', 'MAST', 'DEsingle', 'ballgown',
#' 'NOIseq', 'TCC', 'DEDS', 'cuffdiff', 'fishpond', 'missMethyl', 'DMRcate',
#' 'DEP', 'msmsTests', 'plgem', 'SDAMS', 'DEqMS', 'DiffBind' and 'prescored'. 
#' Where 'prescored' is a dataframe containing the test statistic and gene ID 
#' (either in rowname or separate column) and nothing else. 'preranked' is an 
#' alias for 'prescored'.
#' @param geneIDcol the column containing gene names. If gene names are 
#' specified as row names, then geneIDcol=NULL.
#' @param joinType the type of join to perform, either 'inner' or 'full'.
#' By default, joins are 'inner' except for Seurat and muscat where full is 
#' used.
#' @param geneTable a 2 column table mapping gene identifiers in the profile to
#' gene identifiers in the gene sets. 
#' @return a multi-column table compatible with mitch_calc analysis.
#' @keywords import mitch
#' @export
#' @examples
#' # first step is to create a list of differential profiles
#' data(rna,k9a,k36a)
#' x<-list('rna'=rna,'k9a'=k9a,'k36a'=k36a)
#' # import as edgeR table 
#' imported<-mitch_import(x,DEtype='edger')
#' @importFrom plyr join_all
mitch_import <- function(x, DEtype, geneIDcol = NULL, geneTable = NULL, 
    joinType = NULL) {
    
    if (is.data.frame(x)) {
        message("The input is a single dataframe; one contrast only. Converting
        it to a list for you.")
        NAME <- deparse(substitute(x))
        x <- list(x = x)
    }
    
    if (!is.list(x)) {
        stop("Error: Input (x) must be a LIST of dataframes.")
    }
    
    if (is.null(names(x))) {
        stop("Error: Input (x) must be a NAMED list of dataframes.")
    }

    if ( length(x)>69 ) {
        stop("Error: mitch is currently limited to 69 dimensions or fewer.")
    }
    
    if (!is.null(geneTable) && !is.data.frame(geneTable)) {
        stop("Error: the geneTable needs to be a dataframe.")
    }
    
    if (!is.null(geneTable) && (ncol(geneTable) < 2 || ncol(geneTable) > 2)) {
        stop("Error: the geneTable needs to be a dataframe of 2 columns.")
    }
    
    # the geneIDcol should be an attribute added to each list item
    for (i in seq_len(length(x))) {
        if (!is.null(geneIDcol)) {
            LEN <- length(which(names(x[[i]]) %in% geneIDcol))
            if (LEN < 1) {
                stop("Error: the specified geneIDcol doesn't seem to exist")
            }
            if (LEN > 1) {
                stop("Error: there are multiple matches for the specified
                geneIDcol")
            }
            attributes(x[[i]])$geneIDcol <- which(names(x[[i]]) %in% geneIDcol)
        } else {
            attributes(x[[i]])$geneIDcol <- NULL
        }
        if (!is.null(geneTable)) {
            if (!is.data.frame(geneTable)) {
                stop("Error: geneTable is not a data frame.")
            }
            if (ncol(geneTable) != 2) {
                stop("Error: geneTable must be a 2 column dataframe.")
            }
            attributes(x[[i]])$geneTable <- geneTable
        }
    }
    
    DEtype <- tolower(DEtype)
    
    validDEtype <- c("edger", "deseq2", "limma", "absseq", "sleuth", "seurat",
        "topconfects", "muscat", "swish", "scde", "mast", "desingle",
        "ballgown", "noiseq", "tcc", "deds", "cuffdiff", "preranked", 
        "prescored","fishpond", "missmethyl", "dmrcate", "dep", "msmstests",
        "plgem", "sdams", "deqms", "diffbind")

    if (DEtype == "edger") {
        xx <- lapply(x, edger_score)
    } else if (DEtype == "deseq2" || DEtype == "swish" || 
    DEtype == "fishpond" ) {
        xx <- lapply(x, deseq2_score)
    } else if (DEtype == "absseq") {
        xx <- lapply(x, absseq_score)
    } else if (DEtype == "sleuth") {
        xx <- lapply(x, sleuth_score)
    } else if (DEtype == "seurat") {
        xx <- lapply(x, seurat_score)
    } else if (DEtype == "topconfects") {
        xx <- lapply(x, topconfect_score)
    } else if (DEtype == "muscat") {
        xx <- lapply(x, muscat_score)
    } else if (DEtype == "scde") {
        xx <- lapply(x, scde_score)
    } else if (DEtype == "mast" ) {
        xx <- lapply(x, mast_score)
    } else if (DEtype == "desingle" ) {
        xx <- lapply(x, desingle_score)
    } else if (DEtype == "ballgown" ) {
        xx <- lapply(x, ballgown_score)
    } else if (DEtype == "noiseq" ) {
        xx <- lapply(x, noiseq_score)
    } else if (DEtype == "tcc" ) {
        xx <- lapply(x, tcc_score)
    } else if (DEtype == "deds" || DEtype == "missmethyl" || 
    DEtype == "limma" || DEtype == "deqms" ) {
        xx <- lapply(x, deds_score)
    } else if (DEtype == "cuffdiff" ) {
        xx <- lapply(x, cuffdiff_score)
    } else if (DEtype == "dmrcate" ) {
        xx <- lapply(x, dmrcate_score)
    } else if (DEtype == "dep" ) {
        xx <- lapply(x, dep_score)
    } else if (DEtype == "msmstests" ) {
        xx <- lapply(x, msmstests_score)
    } else if (DEtype == "plgem" ) {
        xx <- lapply(x, plgem_score)
    } else if (DEtype == "sdams" ) {
        xx <- lapply(x, sdams_score)
    } else if (DEtype == "diffbind" ) {
        xx <- lapply(x, diffbind_score)
    } else if (DEtype == "preranked" || DEtype == "prescored") {
        xx <- lapply(x, preranked_score, joinType = joinType, geneIDcol = geneIDcol)
    } else {
        stop(paste("Specified DEtype does not match one of the following:",
        validDEtype))
    }
    
    # give the colums a unique name otherwise join_all will fail
    for (i in seq_len(length(xx))) {
        colnames(xx[[i]]) <- c("geneidentifiers", paste("y", i, sep = ""))
    }
    
    if (is.null(joinType)) {
        if (DEtype == "seurat" || DEtype == "muscat" || DEtype == "scde" ||
        DEtype == "mast" || DEtype == "desingle" ) {
            xxx <- join_all(xx, by = "geneidentifiers", type = "full")
        } else {
            xxx <- join_all(xx, by = "geneidentifiers", type = "inner")
        }
    } else {
        xxx <- join_all(xx, by = "geneidentifiers", type = joinType)
    }
    rownames(xxx) <- xxx$geneidentifiers
    xxx$geneidentifiers <- NULL
    colnames(xxx) <- names(x)
    
    STARTSWITHNUM <- length(grep("^[0-9]", colnames(xxx)))
    if (STARTSWITHNUM > 0) {
        stop("Error: it looks like one or more column names starts with a
        number. This is incompatible with downstream analysis. Please modify")
    }
    
    MEAN_N_GENES_IN <- mean(unlist(lapply(x, nrow)))
    N_GENES_OUT <- nrow(xxx)
    PROP <- signif(N_GENES_OUT/MEAN_N_GENES_IN, 3)
    message(paste("Note: Mean no. genes in input =", MEAN_N_GENES_IN))
    message(paste("Note: no. genes in output =", N_GENES_OUT))
    if (PROP < 0.05) {
        warning("Warning: less than half of the input genes are also in the
        output")
    } else {
        message(paste("Note: estimated proportion of input genes in output =",
        PROP))
    }
    return(xxx)
}

#' gmt_import
#'
#' This function imports GMT files into a list of character vectors for mitch 
#' analysis. GMT files are a commonly used
#' format for lists of genes used in pathway enrichment analysis. GMT files can
#' be obtained from Reactome, MSigDB, etc.
#' @param gmtfile a gmt file.
#' @return a list of gene sets.
#' @keywords import genesets
#' @export
#' @examples
#' # Import some gene sets
#' genesetsExample<-gmt_import(system.file('extdata/sample_genesets.gmt', 
#' package = 'mitch'))
#' @import utils
gmt_import <- function(gmtfile) {
    genesetLines <- strsplit(readLines(gmtfile), "\t")
    genesets <- lapply(genesetLines, utils::tail, -2)
    names(genesets) <- unlist(lapply(genesetLines, head, 1))
    attributes(genesets)$originfile <- gmtfile
    genesets <- genesets[lapply(genesets,length)>0]
    if( any(duplicated(names(genesets))) ) {
        warning("Duplicated gene sets names detected")
    }
    genesets
}

MANOVA <- function(x, genesets, minsetsize = 10, cores = 1,
    priority = NULL) {
    
    STARTSWITHNUM <- length(grep("^[0-9]", colnames(x)))
    if (STARTSWITHNUM > 0) {
        stop("Error: it looks like one or more column names starts with a
        number. This is incompatible with downstream analysis. Please modify")
    }
    
    sets <- names(genesets)
    
    if (is.null(priority)) {
        priority <- "significance"
    }
    
    if (priority !="significance" && priority !="effect" && priority !="SD"){
        stop("Error: Parameter 'priority' must be either 'significance'(the 
        default), 'effect' or 'SD'.")
    }
    
    hypotenuse <- function(x) {
        sqrt(sum(unlist(lapply(x, function(x) { x^2 })),na.rm = TRUE ))
    }
    
    # calculate the hypotenuse for downstream use
    HYPOT <- hypotenuse(apply(x, 2, length))
    
    res <- mclapply(sets, function(set) {
        
        inset <- rownames(x) %in% as.character(unlist(genesets[set]))
        
        NROW <- nrow(x)
        
        if (length(which(inset)) >= minsetsize) {
            fit <- manova(x ~ inset)
            sumMANOVA <- summary.manova(fit)
            sumAOV <- summary.aov(fit)
            pMANOVA <- sumMANOVA$stats[1, "Pr(>F)"]
            raov <- lapply(sumAOV, function(zz) {
                zz[1, "Pr(>F)"]
            })
            raov <- unlist(raov)
            names(raov) <- gsub("^ Response ", "p.", names(raov))
            # S coordinates
            NOTINSET <- colMeans(x[!inset, ], na.rm=TRUE)
            scord <- (2 * (colMeans(x[inset, ], na.rm=TRUE) - NOTINSET))/NROW
            names(scord) <- paste0("s-", names(scord))
            # calculate the hypotenuse length of s scores
            s.dist <- hypotenuse(scord)
            names(s.dist) <- "s.dist"
            mysd <- sd(scord)
            names(mysd) <- "SD"
            
            return(data.frame(set, setSize = sum(inset), pMANOVA, t(scord), 
                t(raov),t(s.dist), t(mysd), stringsAsFactors = FALSE))
        }
    }, mc.cores = cores)
    
    fres <- ldply(res, data.frame)
    
    if (nrow(fres) < 1) {
        
        message("Warning: No results found. Check that the gene names in the
            profile match the gene sets and consider loosening the minsetsize
            parameter.")
        
    } else {
        fres$p.adjustMANOVA <- p.adjust(fres$pMANOVA, "fdr")
        
        # prioritisation
        if (priority == "significance") {
            fres <- fres[order(fres$pMANOVA), ]
            message("Note: When prioritising by significance (ie: small 
            p-values), large effect sizes might be missed.")
        }
        if (priority == "effect") {
            fres <- fres[order(-fres$s.dist), ]
            message("Note: Enrichments with large effect sizes may not be 
            statistically significant.")
        }
        if (priority == "SD") {
            fres <- fres[order(-fres$SD), ]
            fres <- subset(fres, p.adjustMANOVA <= 0.05)
            message("Note: Prioritisation by SD after selecting sets with 
            p.adjustMANOVA<=0.05.")
        }
        attributes(fres)$priority <- priority
        return(fres)
    }
}


ANOVA <- function(x, genesets, minsetsize = 10, cores = 1, 
priority = NULL) {
    
    STARTSWITHNUM <- length(grep("^[0-9]", colnames(x)))
    if (STARTSWITHNUM > 0) {
        stop("Error: it looks like one or more column names starts with a
        number. This is incompatible with downstream analysis. Please modify")
    }
    
    sets <- names(genesets)
    
    if (is.null(priority)) {
        priority <- "significance"
    }
    
    if (priority != "significance" && priority != "effect") {
        stop("Error: Parameter 'priority' must be either 'significance'
        (default) or 'effect'.")
    }

    x <- x[which(!is.na(x)),,drop=FALSE]
    
    res <- mclapply(sets, function(set) {
        resample <- function(x, set) {
            sss <- x[which(rownames(x) %in% 
                as.character(unlist(genesets[set]))),]
            mysample <- sample(sss, length(sss), replace = TRUE)
            mean(mysample)
        }
        
        inset <- rownames(x) %in% as.character(unlist(genesets[set]))
        
        NROW <- nrow(x)
        
        if (length(which(inset)) >= minsetsize) {
            fit <- aov(x[, 1] ~ inset)
            pANOVA <- summary(fit)[[1]][, 5][1]
            NOTINSET <- mean(x[!inset, ])
            s.dist <- (2 * (mean(x[inset, ]) - NOTINSET))/NROW
            gres <- data.frame(set, setSize = sum(inset), pANOVA, s.dist,
            stringsAsFactors = FALSE)
            gres
        }
    }, mc.cores = cores)
    
    fres <- ldply(res, data.frame)
    
    if (nrow(fres) < 1) {
        
        message("Warning: No results found. Check that the gene names in the
        profile match the gene sets and consider loosening the minsetsize
        parameter.")
        
    } else {
        fres$p.adjustANOVA <- p.adjust(fres$pANOVA, "fdr")
        
        # prioritisation
        if (priority == "significance") {
            fres <- fres[order(fres$pANOVA), ]
            message("Note: When prioritising by significance (ie: small
            p-values), large effect sizes might be missed.")
        }
        if (priority == "effect") {
            fres <- fres[order(-abs(fres$s.dist)), ]
            message("Note: Enrichments with large effect sizes may not be
            statistically significant.")
        }
        attributes(fres)$priority <- priority
        return(fres)
    }
}

mitch_metrics_calc<-function(x, genesets, enrichment_result, minsetsize = 10) {
    
    if (!is.null(enrichment_result)) {
        
        num_genesets <- length(genesets)
        included_genesets <- nrow(enrichment_result)
        geneset_counts <- as.data.frame(as.vector(unlist(lapply(genesets, 
        function(set) {
            length(which(as.vector(unlist(set)) %in% rownames(x)))
        }))))
        rownames(geneset_counts) <- names(genesets)
        colnames(geneset_counts) <- "count"
        genesets_excluded <-
            names(genesets)[which(geneset_counts$count<minsetsize)]
        genesets_included <-
            names(genesets)[which(geneset_counts$count>=minsetsize)]
        num_genesets_excluded <- length(genesets_excluded)
        num_genesets_included <- length(genesets_included)
        num_genes_in_genesets <- length(unique(as.vector(unlist(genesets))))
        num_genes_in_profile <- length(unique(rownames(x)))
        duplicated_genes_present <- length(rownames(x)) > num_genes_in_profile
        num_profile_genes_in_sets <- length(which(rownames(x) %in% 
        as.vector(unlist(genesets))))
        num_profile_genes_not_in_sets <- num_genes_in_profile - 
        num_profile_genes_in_sets
        num_sets_significant <- 
            nrow(enrichment_result[which(enrichment_result$p.adjustMANOVA < 
            0.05), ])
        profile_pearson_correl <- cor(x, method = "p")[2, 1]
        profile_spearman_correl <- cor(x, method = "s")[2, 1]
        
        # genes in each quadrant
        g1 <- length(which(x[, 1] > 0 & x[, 2] > 0))
        g2 <- length(which(x[, 1] > 0 & x[, 2] < 0))
        g3 <- length(which(x[, 1] < 0 & x[, 2] < 0))
        g2 <- length(which(x[, 1] < 0 & x[, 2] > 0))
        
        # genesets in each quadrant
        ns1 <- nrow(subset(enrichment_result, p.adjustMANOVA < 0.05 &
            enrichment_result[,4] > 0 & enrichment_result[, 5] > 0))
        ns2 <- nrow(subset(enrichment_result, p.adjustMANOVA < 0.05 & 
            enrichment_result[,4] > 0 & enrichment_result[, 5] < 0))
        ns3 <- nrow(subset(enrichment_result, p.adjustMANOVA < 0.05 &
            enrichment_result[,4] < 0 & enrichment_result[, 5] < 0))
        ns4 <- nrow(subset(enrichment_result, p.adjustMANOVA < 0.05 &
            enrichment_result[,4] < 0 & enrichment_result[, 5] > 0))
        num_sets_significant_by_quadrant <- paste(ns1, ns2, ns3, ns4, sep = ",")
        
        dat <- list(num_genesets = num_genesets, 
            num_genes_in_profile = num_genes_in_profile, 
            duplicated_genes_present = duplicated_genes_present, 
            num_profile_genes_in_sets = num_profile_genes_in_sets, 
            num_profile_genes_not_in_sets = num_profile_genes_not_in_sets,
            num_genesets_excluded = num_genesets_excluded, 
            num_genesets_included = num_genesets_included, 
            num_genes_in_genesets = num_genes_in_genesets, 
            genesets_excluded = genesets_excluded,
            genesets_included = genesets_included, 
            profile_pearson_correl = profile_pearson_correl,
            profile_spearman_correl = profile_spearman_correl, 
            num_sets_significant = num_sets_significant,
            num_sets_significant_by_quadrant=num_sets_significant_by_quadrant,
            geneset_counts = geneset_counts)
        dat
    }
}

mitch_metrics_calc1d <- function(x, genesets, anova_result, minsetsize = 10) {
    
    if (!is.null(anova_result)) {
        
        num_genesets <- length(genesets)
        included_genesets <- nrow(anova_result)
        geneset_counts <- as.data.frame(as.vector(unlist(lapply(genesets,
            function(set) {
            length(which(as.vector(unlist(set)) %in% rownames(x)))
        }))))
        rownames(geneset_counts) <- names(genesets)
        colnames(geneset_counts) <- "count"
        genesets_excluded <-
            names(genesets)[which(geneset_counts$count<minsetsize)]
        genesets_included <-
            names(genesets)[which(geneset_counts$count >= minsetsize)]
        num_genesets_excluded <- length(genesets_excluded)
        num_genesets_included <- length(genesets_included)
        num_genes_in_genesets <- length(unique(as.vector(unlist(genesets))))
        num_genes_in_profile <- length(unique(rownames(x)))
        duplicated_genes_present <- length(rownames(x)) > num_genes_in_profile
        num_profile_genes_in_sets <- length(which(rownames(x) %in% 
            as.vector(unlist(genesets))))
        num_profile_genes_not_in_sets <- num_genes_in_profile-num_profile_genes_in_sets
        num_sets_significant <-
            nrow(anova_result[which(anova_result$p.adjustANOVA<0.05),])
        
        # genes up and down
        g1 <- length(which(x[, 1] > 0))
        g2 <- length(which(x[, 1] < 0))
        
        # genesets in each quadrant
        num_sets_up <- nrow(subset(anova_result, p.adjustANOVA < 0.05 &
            anova_result[,4] > 0))
        num_sets_dn <- nrow(subset(anova_result, p.adjustANOVA < 0.05 &
            anova_result[,4] < 0))
        
        dat <- list(num_genesets = num_genesets,
            num_genes_in_profile = num_genes_in_profile, 
            duplicated_genes_present = duplicated_genes_present,
            num_profile_genes_in_sets = num_profile_genes_in_sets, 
            num_profile_genes_not_in_sets = num_profile_genes_not_in_sets,
            num_genesets_excluded = num_genesets_excluded, 
            num_genesets_included = num_genesets_included,
            num_genes_in_genesets = num_genes_in_genesets, 
            genesets_excluded = genesets_excluded,
            genesets_included = genesets_included, 
            num_sets_significant = num_sets_significant,
            num_sets_up = num_sets_up, 
            num_sets_dn = num_sets_dn,
            geneset_counts = geneset_counts)
        dat
    }
}


mitch_rank <- function(x) {
    
    for (i in seq_len(ncol(x))) {
        LEN <- length(x[, i])
        UNIQLEN <- length(unique(x[, i]))
        if (UNIQLEN/LEN < 0.4) {
            warning("Warning: >60% of genes have the same score. This isn't
            optimal for rank based enrichment analysis.")
        }
    }
    
    rank_adj <- function(x) {
        xx <- rank(x,na.last = "keep")
        num_neg <- length(which(x < 0))
        num_zero <- length(which(x == 0))
        num_adj <- num_neg + (num_zero/2)
        adj <- xx - num_adj
        adj
    }
    adj <- apply(x, 2, rank_adj)
    adj
}


detailed_sets <- function(res, resrows = 50) {
    # collect ranked genelist of each genest
    genesets <- res$input_genesets
    ss <- res$ranked_profile
    mykeys <- as.character(res$enrichment_result[seq_len(resrows), 1])
    dat <- vector(mode = "list", length = resrows)
    names(dat) <- mykeys
    
    for (i in seq_len(resrows)) {
        sss <- ss[which(rownames(ss) %in% genesets[[which(names(genesets) %in%
        as.character(res$enrichment_result[i, 
            1]))]]), ]
        dat[[i]] <- sss
    }
    dat
}


get_os <- function(){
    sysinf <- Sys.info()
    if (!is.null(sysinf)){
        os <- sysinf['sysname']
        if (os == 'Darwin')
            os <- "osx"
    } else { ## mystery machine
        os <- .Platform$OS.type
        if (grepl("^darwin", R.version$os))
            os <- "osx"
        if (grepl("linux-gnu", R.version$os))
            os <- "linux"
    }
    tolower(os)
}


#' mitch_calc
#'
#' This function performs multivariate gene set enrichment analysis. 
#' @param x a multicolumn numerical table with each column containing 
#' differential expression scores for a contrast.
#' Rownames must match genesets.
#' @param genesets lists of genes imported by the gmt_imprt function or
#' similar.
#' @param minsetsize the minimum number of genes required in a set for it to be
#' included in the statistical analysis.
#' Default is 10.
#' @param cores the number of parallel threads for computation. Defaults to 1.
#' @param resrows an integer value representing the number of top genesets for
#' which a detailed report is to be 
#' generated. Default is 50.
#' @param priority the prioritisation metric used to selecting top gene sets. 
#' Valid options are 'significance', 
#' 'effect' and 'SD'. 
#' @return mitch res object with the following parts:
#' $input_profile: the supplied input differential profile
#' $input_genesets: the supplied input gene sets
#' $ranked_profile: the differential profile after ranking
#' $enrichment_result: the table of MANOVA/ANOVA enrichment results for each
#' gene set
#' $analysis_metrics:  several metrics that are important to the interpretation
#' of the results
#' $detailed_sets: a list of dataframes containing ranks of members of
#' prioritised gene sets.
#' @keywords mitch calc calculate manova 
#' @import parallel
#' @import stats
#' @importFrom plyr ldply
#' @export
#' @examples
#' # Example using mitch to calculate multivariate enrichments and
#' # prioritise based on effect size 
#' data(myImportedData,genesetsExample)
#' resExample<-mitch_calc(myImportedData,genesetsExample,priority='effect',
#' minsetsize=5,cores=2)
mitch_calc <- function(x, genesets, minsetsize = 10, cores = 1,
    resrows = 50, priority = NULL) {
    
    colnames(x) <- gsub("[[:punct:]]", "_", colnames(x))
    colnames(x) <- substr(colnames(x), 1, 14)
    if ( any(duplicated(colnames(x)))) { stop("Duplicate column names.") }
    input_profile <- x
    input_genesets <- genesets
    ranked_profile <- mitch_rank(input_profile)
    if (get_os() == "windows") { cores <- 1 }
    
    if ( ncol(x)>69 ) {
        stop("Error: mitch is currently limited to 69 dimensions or fewer.")
    }

    if (ncol(x) > 1) {

    corx <- cor(x,use="pairwise.complete.obs")
    diag(corx) <- 0

        if ( max(corx) == 1 ) {
            stop("Error: It looks like two or more contrasts are identical. Aborting.")
        }

        enrichment_result <- MANOVA(ranked_profile, genesets, 
            minsetsize <- minsetsize, cores = cores, priority = priority)
        
        if (!is.null(enrichment_result)) {
            mitch_metrics <- mitch_metrics_calc(x, genesets, enrichment_result)
            dat <- list(input_profile = input_profile,
                input_genesets = input_genesets,
                ranked_profile = ranked_profile,
                enrichment_result = enrichment_result, 
                analysis_metrics = mitch_metrics)
            
            if (nrow(enrichment_result) < resrows) {
                resrows <- nrow(enrichment_result)
            }
            dat$detailed_sets <- detailed_sets(dat, resrows)
            attr(dat, "profile_dimensions") <- colnames(dat$input_profile)
            dat
        }
    } else if (ncol(x) == 1) {
        
        enrichment_result <- ANOVA(ranked_profile, genesets,
            minsetsize <- minsetsize, cores = cores, priority = priority)
        
        if (!is.null(enrichment_result)) {
            mitch_metrics <- mitch_metrics_calc1d(x, genesets,
            enrichment_result)
            dat <- list(input_profile = input_profile,
                input_genesets = input_genesets, 
                ranked_profile = ranked_profile,
                enrichment_result = enrichment_result, 
                analysis_metrics = mitch_metrics)
            
            if (nrow(enrichment_result) < resrows) {
                resrows <- nrow(enrichment_result)
            }
            dat$detailed_sets <- detailed_sets(dat, resrows)
            attr(dat, "profile_dimensions") <- colnames(dat$input_profile)
            dat
        }
    }
}

plot1d_profile_dist <- function(res) {
    par(mfrow = c(2, 1))
    hist(res$input_profile[, 1], breaks = 50,
        main = "Distribution of DE scores", xlab = paste("DE score for ",
        colnames(res$input_profile)))
    plot(res$input_profile, xlab = paste("DE score for ",
        colnames(res$input_profile)), 
        pch = "|", frame.plot = FALSE)
    UPS <- length(which(res$input_profile > 0))
    DNS <- length(which(res$input_profile < 0))
    TOTAL <- nrow(res$input_profile)
    mtext(paste(TOTAL, "genes in total,", UPS, "trending up-regulated,", 
    DNS, "trending down-regulated"))
    pl <- recordPlot()
    pl
}

plot_geneset_hist <- function(res) {
    par(mfrow = c(3, 1))
    geneset_counts <- res$analysis_metrics$geneset_counts
    boxplot(geneset_counts$count, horizontal = TRUE, frame = FALSE, 
        main = "Gene set size", 
        xlab = "number of member genes included in profile")
    hist(geneset_counts$count, 100, xlab = "geneset size",
        main = "Histogram of geneset size")
    hist(geneset_counts$count, 100, xlim = c(0, 500), xlab = "geneset size",
        main = "Trimmed histogram of geneset size")
    pl <- recordPlot()
    pl
}

plot1d_volcano <- function(res) {
    par(mfrow = c(1, 1))
    sig <- subset(res$enrichment_result, p.adjustANOVA <= 0.05)
    plot(res$enrichment_result$s.dist, -log10(res$enrichment_result$pANOVA),
        xlab = "s score", ylab = "-log10(p-value)",
        main = "volcano plot of gene set enrichments",
        pch = 19, cex = 0.8)
    points(sig$s.dist, -log10(sig$pANOVA), pch = 19, cex = 0.85, col = "red")
    TOTAL <- nrow(res$enrichment_result)
    SIG <- nrow(sig)
    UP <- length(which(sig$s.dist > 0))
    DN <- length(which(sig$s.dist < 0))
    SUBHEADER <- paste(TOTAL, "gene sets in total,", UP, "upregulated and ",
        DN, "downregulated (FDR<=0.05)")
    mtext(SUBHEADER)
    pl <- recordPlot()
    pl
}

plot1d_detailed <- function(res, i) {
    par(mfrow = c(3, 1))
    ss <- res$ranked_profile
    sss <- res$detailed_sets[[i]]
    set <- names(res$detailed_sets[i])
    size <- length(sss)
    
    beeswarm(sss, vertical = FALSE, cex = 0.75, xlim = c(min(ss), max(ss)),
        col = "darkgray", pch = 19, main = set, cex.main = 1.5,
        xlab = paste("ranked DE score in:", colnames(ss)))
    mtext("beeswarm plot", cex = 0.8)
    
    hist(sss, xlim = c(min(ss), max(ss)), breaks = 15, col = "darkgray", 
        main = NULL, border = "black", 
        xlab = paste("ranked DE score in:", colnames(ss)))
    mtext("histogram", cex = 0.8)
    
    plot(sss, rep(1, length(sss)), type = "n", xlim = c(min(ss), max(ss)), 
        frame = FALSE, axes = FALSE, ylab = "", 
        xlab = paste("ranked DE score in:", colnames(ss)))
    rug(sss, ticksize = 0.9)
    axis(1)
    mtext("rugplot", cex = 0.8)
    pl <- recordPlot()
    pl
}

plot2d_profile_dist <- function(res) {
    plot(res$input_profile, pch = 19, 
        col = rgb(red = 0, green = 0, blue = 0, alpha = 0.2), 
        main = "Scatterplot of all genes")
    abline(v = 0, h = 0, lty = 2, lwd = 2, col = "blue")
    pl <- recordPlot()
    pl
}


plot2d_profile_density <- function(res) {
    palette <- colorRampPalette(c("white", "yellow", "orange", "red", 
        "darkred", "black"))
    
    ss <- res$ranked_profile
    xmin <- min(ss[, 1])
    xmax <- max(ss[, 1])
    ymin <- min(ss[, 2])
    ymax <- max(ss[, 2])
    
    k <- MASS::kde2d(ss[, 1], ss[, 2])
    X_AXIS <- paste("Rank in contrast", colnames(ss)[1])
    Y_AXIS <- paste("Rank in contrast", colnames(ss)[2])
    
    filled.contour(k, xlim = c(xmin, xmax), ylim = c(ymin, ymax), 
        color.palette = palette, plot.title = {
            abline(v = 0, h = 0, lty = 2, lwd = 2, col = "blue")
            title(main = "Rank-rank plot of all genes", xlab = X_AXIS, 
            ylab = Y_AXIS)
        })
    pl <- recordPlot()
    pl
}

plot2d_gene_quadrant_barchart <- function(res) {
    uu <- length(which(res$input_profile[, 1] > 0 & res$input_profile[, 2] > 0))
    ud <- length(which(res$input_profile[, 1] > 0 & res$input_profile[, 2] < 0))
    dd <- length(which(res$input_profile[, 1] < 0 & res$input_profile[, 2] < 0))
    du <- length(which(res$input_profile[, 1] < 0 & res$input_profile[, 2] > 0))
    a <- as.data.frame(c(uu, ud, dd, du))
    rownames(a) <- c("top-right", "bottom-right", "bottom-left", "top-left")
    colnames(a) <- "a"
    barplot(a$a, names.arg = rownames(a), 
        main = "number of genes in each quadrant")
    pl <- recordPlot()
    pl
}

plot2d_set_quadrant_barchart <- function(res) {
    par(mfrow = c(1, 1))
    a <- res$analysis_metrics[14]
    a <- as.data.frame(as.numeric(unlist(strsplit(as.character(a), ","))), 
    stringsAsFactors = FALSE)
    rownames(a) <- c("top-right", "bottom-right", "bottom-left", "top-left")
    colnames(a) <- "a"
    barplot(a$a, names.arg = rownames(a), main = "number of genesets FDR<0.05")
    pl <- recordPlot()
    pl
}


plot2d_set_scatter <- function(res) {
    sig <- subset(res$enrichment_result, p.adjustMANOVA < 0.05)
    plot(res$enrichment_result[, 4:5], pch = 19, col = rgb(red = 0, green = 0, 
        blue = 0, alpha = 0.2),
        main = "Scatterplot of all gene sets; FDR<0.05 in red")
    abline(v = 0, h = 0, lty = 2, lwd = 2, col = "blue")
    points(sig[, 4:5], pch = 19, col = rgb(red = 1, green = 0, blue = 0,
    alpha = 0.5))
    pl <- recordPlot()
    pl
}

plot2d_set_scatter_top <- function(res) {
    resrows <- length(res$detailed_sets)
    top <- head(res$enrichment_result, resrows)
    plot(res$enrichment_result[, 4:5], pch = 19, col = rgb(red = 0, green = 0,
        blue = 0, alpha = 0.2),
        main = paste("Scatterplot of all gene sets; top", resrows, 
        "in red"))
    abline(v = 0, h = 0, lty = 2, lwd = 2, col = "blue")
    points(top[, 4:5], pch = 19, col = rgb(red = 1, green = 0, blue = 0,
    alpha = 0.5))
    pl <- recordPlot()
    pl
}

plot2d_heatmap <- function(res) {
    d <- ncol(res$input_profile)
    resrows <- length(res$detailed_sets)
    pl <- NULL
    if (resrows > 2) {
        hmapx <- head(res$enrichment_result[, 4:(4 + d - 1)], resrows)
        rownames(hmapx) <- head(res$enrichment_result$set, resrows)
        colnames(hmapx) <- gsub("^s.", "", colnames(hmapx))
        my_palette <- colorRampPalette(c("blue", "white", "red"))(n = 25)
        heatmap.2(as.matrix(hmapx), scale = "none", margins = c(10, 25),
            cexRow = 0.8, trace = "none", cexCol = 0.8, col = my_palette)
        pl <- recordPlot()
    }
    pl
}

plot_effect_vs_significance <- function(res) {
    par(mfrow = c(1, 1))
    plot(res$enrichment_result$s.dist,
        -log(res$enrichment_result$p.adjustMANOVA), 
        xlab = "s.dist (effect size)", 
        ylab = "-log(p.adjustMANOVA) (significance)", 
        pch = 19, col = rgb(red = 0, green = 0, blue = 0, alpha = 0.2),
        main = "effect size versus statistical significance")
    pl <- recordPlot()
    pl
}


plot2d_detailed_density <- function(res, i) {
    palette <- colorRampPalette(c("white", "yellow", "orange", "red",
        "darkred", "black"))
    ss <- res$ranked_profile
    xmin <- min(ss[, 1])
    xmax <- max(ss[, 1])
    ymin <- min(ss[, 2])
    ymax <- max(ss[, 2])
    ll <- res$enrichment_result[i, ]
    size <- ll$setSize
    sss <- res$detailed_sets[[i]]
    X_AXIS <- paste("Rank in contrast", colnames(ss)[1])
    Y_AXIS <- paste("Rank in contrast", colnames(ss)[2])
    par(mar = c(5, 4, 4, 2))
    k <- MASS::kde2d(sss[, 1], sss[, 2])
    filled.contour(k, color.palette = palette, xlim = c(xmin, xmax),
        ylim = c(ymin, ymax), plot.title = {
        abline(v = 0, h = 0, lty = 2, lwd = 2, col = "blue")
        title(main = ll$set, xlab = X_AXIS, ylab = Y_AXIS)
    })
    pl <- recordPlot()
    pl
}


plot2d_detailed_scatter <- function(res, i) {
    ss <- res$ranked_profile
    xmin <- min(ss[, 1])
    xmax <- max(ss[, 1])
    ymin <- min(ss[, 2])
    ymax <- max(ss[, 2])
    sss <- res$detailed_sets[[i]]
    X_AXIS <- paste("Rank in contrast", colnames(ss)[1])
    Y_AXIS <- paste("Rank in contrast", colnames(ss)[2])
    ll <- res$enrichment_result[i, ]
    plot(sss, pch = 19, col = rgb(red = 0, green = 0, blue = 0, alpha = 0.2),
        main = ll$set, xlim = c(xmin, xmax), ylim = c(ymin, ymax), 
        xlab = X_AXIS, ylab = Y_AXIS)
    abline(v = 0, h = 0, lty = 2, lwd = 2, col = "blue")
    pl <- recordPlot()
    pl
}


plot2d_detailed_violin <- function(res, i) {
    pl <- list()
    
    ss <- res$ranked_profile
    xmin <- min(ss[, 1])
    xmax <- max(ss[, 1])
    ymin <- min(ss[, 2])
    ymax <- max(ss[, 2])
    ll <- res$enrichment_result[i, ]
    size <- ll$setSize
    sss <- res$detailed_sets[[i]]
    X_AXIS <- paste("Rank in contrast", colnames(ss)[1])
    Y_AXIS <- paste("Rank in contrast", colnames(ss)[2])
    ss_long <- melt(ss)
    colnames(ss_long) <- c("gene","contrast","value")
    sss_long <- melt(sss)
    colnames(sss_long) <- c("gene","contrast","value")
    
    p <- ggplot(ss_long, aes(contrast, value)) + geom_violin(data = ss_long,
        fill = "grey", colour = "grey") + geom_boxplot(data = ss_long, 
        width = 0.9, fill = "grey", outlier.shape = NA, 
        coef = 0) + geom_violin(data = sss_long, fill = "black", 
        colour = "black") + geom_boxplot(data = sss_long, width = 0.1, 
        outlier.shape = NA) + labs(y = "Position in rank", title = ll[, 1])
    
    print(p + theme_bw() + theme(axis.text = element_text(size = 14), 
        axis.title = element_text(size = 15), 
        plot.title = element_text(size = 20)))
    pl <- recordPlot()
    pl
}


ggpairs_points <- function(res) {
    ggpairs_points_plot <- function(data, mapping, ...) {
        p <- ggplot(data = data, mapping = mapping) + 
            geom_point(alpha = 0.05) + 
            geom_vline(xintercept = 0, linetype = "dashed") +
            geom_hline(yintercept = 0, 
            linetype = "dashed")
    }
    
    p <- ggpairs(as.data.frame(res$input_profile),
        title = "Scatterplot of all genes", 
        lower = list(continuous = ggpairs_points_plot))
    print(p + theme_bw())
}


ggpairs_points_subset <- function(res) {
    d <- ncol(res$ranked_profile)
    ggpairs_points_plot <- function(data, mapping, ...) {
        p <- ggplot(data = data, mapping = mapping) +
            geom_point(alpha = 0.05) + 
            geom_vline(xintercept = 0, linetype = "dashed") +
            geom_hline(yintercept = 0, 
            linetype = "dashed")
    }
    enrichment_result_clipped <- res$enrichment_result[, 4:(3 + d)]
    colnames(enrichment_result_clipped) <- colnames(res$input_profile)
    p <- ggpairs(enrichment_result_clipped,
        title = "Scatterplot of all genessets; FDR<0.05 in red", 
        lower = list(continuous = ggpairs_points_plot))
    print(p + theme_bw())
}


ggpairs_contour <- function(res) {
    palette <- colorRampPalette(c("white", "yellow", "orange", "red",
        "darkred", "black"))
    ggpairs_func <- function(data, mapping, ...) {
        p <- ggplot(data = data, mapping = mapping) +
            stat_density2d(aes(fill = ..density..), 
            geom = "tile", contour = FALSE) +
            geom_vline(xintercept = 0, linetype = "dashed") + 
            geom_hline(yintercept = 0, linetype = "dashed") +
            scale_fill_gradientn(colours = palette(25))
        p
    }
    ss <- res$ranked_profile
    p <- ggpairs(as.data.frame(ss),
        title = "Contour plot of all genes after ranking", 
        lower = list(continuous = ggpairs_func),
            diag = list(continuous = wrap("barDiag", 
            binwidth = nrow(ss)/100)))
    print(p + theme_bw())
}


colname_substitute <- function(res) {
    d <- ncol(res$ranked_profile)
    if (d > 5) {
        mydims <- data.frame(attributes(res)$profile_dimensions)
        colnames(mydims) <- "dimensions"
        colnames(res$input_profile) <- paste("d",
            seq_len(ncol(res$input_profile)), sep = "")
        colnames(res$ranked_profile) <- paste("d",
            seq_len(ncol(res$ranked_profile)), sep = "")
    }
    res
}


gene_sector_table <- function(res) {
    mytheme <- gridExtra::ttheme_default(
        core = list(fg_params = list(cex = 0.5)), 
        colhead = list(fg_params = list(cex = 0.7)),
        rowhead = list(fg_params = list(cex = 0.7)))
    
    d <- ncol(res$ranked_profile)
    ss <- res$ranked_profile
    sig <- sign(ss)
    if (d < 6) {
        sig <- sign(ss)
        sector_count <- aggregate(seq(from = 1, to = nrow(sig)) ~ ., 
            sig, FUN = length)
        colnames(sector_count)[ncol(sector_count)] <- 
            "Number of genes in each sector"
        grid.newpage()
        grid.table(sector_count, theme = mytheme)
    }
}


geneset_sector_table <- function(res) {
    d <- ncol(res$ranked_profile)
    mytheme <- gridExtra::ttheme_default(
        core = list(fg_params = list(cex = 0.5)), 
        colhead = list(fg_params = list(cex = 0.7)),
        rowhead = list(fg_params = list(cex = 0.7)))
    
    sig <- 
        sign(res$enrichment_result[which(res$enrichment_result$p.adjustMANOVA< 
        0.05), 4:(4 + d - 1)])
    if (d < 6) {
        if (nrow(sig) > 0) {
            sector_count <- aggregate(seq(from = 1, to = nrow(sig)) ~ ., 
                sig, FUN = length)
            colnames(sector_count)[ncol(sector_count)] <-
                "Number of gene sets in each sector"
            grid.newpage()
            grid.table(sector_count, theme = mytheme)
        }
    }
}

heatmapx <- function(res) {
    d <- ncol(res$ranked_profile)
    resrows <- length(res$detailed_sets)
    hmapx <- head(res$enrichment_result[, 4:(4 + d - 1)], resrows)
    rownames(hmapx) <- head(res$enrichment_result$set, resrows)
    colnames(hmapx) <- gsub("^s.", "", colnames(hmapx))
    my_palette <- colorRampPalette(c("blue", "white", "red"))(n = 25)
    heatmap.2(as.matrix(hmapx), scale = "none", margins = c(10, 25),
        cexRow = 0.8, trace = "none", cexCol = 0.8, col = my_palette)
    pl <- recordPlot()
    pl
}

plot3d_detailed_density <- function(res, i) {
    palette <- colorRampPalette(c("white", "yellow", "orange", "red",
        "darkred", "black"))
    d <- ncol(res$ranked_profile)
    ss <- res$ranked_profile
    ll <- res$enrichment_result[i, ]
    size <- ll$setSize
    sss <- res$detailed_sets[[i]]
    
    empty_cnt <- length(which(is.na(cor(sss,use="pairwise.complete.obs"))))
    if ( empty_cnt > 0 ) { return("Too few genes. Skipping density plot.") }

    if (d > 5) {
        colnames(sss) <- paste("d", seq_len(ncol(res$input_profile)), sep = "")
    }
    
    ggpairs_contour_limit_range <- function(data, mapping, ...) {
        p <- ggplot(data = data, mapping = mapping) +
            stat_density2d(aes(fill = ..density..), 
            geom = "tile", contour = FALSE) +
            geom_vline(xintercept = 0, linetype = "dashed") + 
            geom_hline(yintercept = 0, linetype = "dashed") +
            scale_fill_gradientn(colours = palette(25)) + 
            scale_x_continuous(limits = 
                range(min(ss[, gsub("~", "", as.character(mapping[1]))]), 
                max(ss[, gsub("~", "", 
                as.character(mapping[1]))]))) +
            scale_y_continuous(limits = range(min(ss[, 
            gsub("~", "", as.character(mapping[2]))]),
            max(ss[, gsub("~", "", as.character(mapping[2]))])))
        p
    }
    
    p <- ggpairs(as.data.frame(sss), title = ll[, 1],
        lower = list(continuous = ggpairs_contour_limit_range), 
        diag = list(continuous = wrap("barDiag", binwidth = nrow(ss)/10)))
    print(p + theme_bw())
}


plot3d_detailed_points <- function(res, i) {
    d <- ncol(res$ranked_profile)
    ss <- res$ranked_profile
    ll <- res$enrichment_result[i, ]
    size <- ll$setSize
    sss <- res$detailed_sets[[i]]

    empty_cnt <- length(which(is.na(cor(sss,use="pairwise.complete.obs"))))
    if ( empty_cnt > 0 ) { return("Too few genes. Skipping ggpairs plot.") }
    
    if (d > 5) {
        colnames(sss) <- paste("d", seq_len(ncol(res$input_profile)), sep = "")
    }
    
    ggpairs_points_limit_range <- function(data, mapping, ...) {
        p <- ggplot(data = data, mapping = mapping) +
            geom_point(alpha = 0.1) +
            geom_vline(xintercept = 0, linetype = "dashed") +
            geom_hline(yintercept = 0, linetype = "dashed") + 
            scale_x_continuous(limits =
                range(min(ss[, gsub("~", "", as.character(mapping[1]))]), 
                max(ss[, gsub("~", "", as.character(mapping[1]))]))) +
            scale_y_continuous(limits = range(min(ss[, 
            gsub("~", "", as.character(mapping[2]))]), 
            max(ss[, gsub("~", "", as.character(mapping[2]))])))
        p
    }
    
    p <- ggpairs(as.data.frame(sss), title = ll[, 1],
        lower = list(continuous = ggpairs_points_limit_range), 
        diag = list(continuous = wrap("barDiag", binwidth = nrow(ss)/10)))
    print(p + theme_bw())
}


plot3d_detailed_violin <- function(res, i) {
    d <- ncol(res$ranked_profile)
    ss <- res$ranked_profile
    ll <- res$enrichment_result[i, ]
    sss <- res$detailed_sets[[i]]

    empty_cnt <- apply(sss,2,function(x) sum(as.numeric(is.finite(x))))
    empty_cnt <- length(which(empty_cnt==0))
    if ( empty_cnt > 0 ) { return("Too few genes. Skipping violin plot.") }
    
    if (d > 5) {
        colnames(sss) <- paste("d", seq_len(ncol(res$input_profile)), sep = "")
    }
    ss_long <- melt(ss)
colnames(ss_long) <- c("gene","contrast","value")
    sss_long <- melt(sss)
colnames(sss_long) <- c("gene","contrast","value")
    p <- ggplot(ss_long, aes(contrast, value)) +
        geom_violin(data = ss_long, fill = "grey", colour = "grey") +
        geom_boxplot(data = ss_long, width = 0.9, fill = "grey", 
        outlier.shape = NA, coef = 0) +
        geom_violin(data = sss_long, fill = "black", colour = "black") +
        geom_boxplot(data = sss_long, width = 0.1, outlier.shape = NA) + 
        labs(y = "Position in rank", title = ll[, 1])
    
    print(p + theme_bw() +
        theme(axis.text = element_text(size = 14),
        axis.title = element_text(size = 15), 
        plot.title = element_text(size = 20)))
}


#' mitch_plots
#'
#' This function generates several plots of multivariate gene set enrichment in
#' high resolution PDF format.
#' The number of detailed sets to generate is dictated by the resrows set in
#' the mitch_calc command.
#' @param res a mitch results object.
#' @param outfile the destination file for the plots in PDF format. should
#' contain 'pdf' suffix. Defaults to 
#' 'Rplots.pdf'
#' @return generates a PDF file containing enrichment plots.
#' @keywords mitch plot plots pdf 
#' @export
#' @examples
#' data(resExample)
#' mitch_plots(resExample,outfile='outres.pdf')
#' @import grDevices
#' @import graphics
#' @import GGally
#' @import grid
#' @import gridExtra
#' @importFrom beeswarm beeswarm
#' @importFrom gplots heatmap.2
#' @importFrom reshape2 melt
#' @import ggplot2
#' @importFrom MASS kde2d
mitch_plots <- function(res, outfile = "Rplots.pdf") {
    
    resrows <- length(res$detailed_sets)
    d <- ncol(res$ranked_profile)
    pdf(outfile)
    
    if ( d>20 ) {
        stop("Error: mitch plotting features are impractical for over 20
        dimensions.")
    }

    if (d == 1) {
        
        plot1d_profile_dist(res)
        plot_geneset_hist(res)
        plot1d_volcano(res)
        lapply(seq_len(resrows), function(i) {
            plot1d_detailed(res, i)
        })
        
    } else if (d == 2) {
        
        plot2d_profile_dist(res)
        plot2d_profile_density(res)
        plot2d_gene_quadrant_barchart(res)
        plot_geneset_hist(res)
        plot2d_set_quadrant_barchart(res)
        plot2d_set_scatter(res)
        plot2d_set_scatter_top(res)
        plot2d_heatmap(res)
        plot_effect_vs_significance(res)
        
        lapply(seq_len(resrows), function(i) {
            plot2d_detailed_density(res, i)
            plot2d_detailed_scatter(res, i)
            plot2d_detailed_violin(res, i)
        })
    } else if (d > 2) {
        
        res <- colname_substitute(res)
        ggpairs_points(res)
        ggpairs_contour(res)
        gene_sector_table(res)
        plot_geneset_hist(res)
        geneset_sector_table(res)
        ggpairs_points_subset(res)
        heatmapx(res)
        plot_effect_vs_significance(res)
        
        lapply(seq_len(resrows), function(i) {
            plot3d_detailed_density(res, i)
            plot3d_detailed_points(res, i)
            plot3d_detailed_violin(res, i)
        })
    }
    dev.off()
}


#' mitch_report
#'
#' This function generates an R markdown based html report containing tables
#' and several plots of mitch results 
#' The plots are in png format, so are not as high in resolution as compared to
#' the PDF generated by mitch_plots 
#' function. The number of detailed sets to generate is dictated by the resrows
#' set in the mitch_calc command.
#' @param res a mitch results object.
#' @param outfile the destination file for the html report. should contain
#' 'html' suffix. Defaults to 
#' 'Rplots.pdf'
#' @param overwrite should overwrite the report file if it already exists?
#' @return generates a HTML file containing enrichment plots.
#' @keywords mitch report html markdown knitr
#' @export
#' @examples
#' data(resExample)
#' mitch_report(resExample,'outres2.html')
#' @import knitr
#' @importFrom rmarkdown render
#' @importFrom dplyr mutate
#' @import echarts4r
#' @import kableExtra
mitch_report <- function(res, outfile , overwrite=FALSE) {

    df <- data.frame(dummy_x = seq(20), dummy_y = rnorm(20, 10, 3))
    trash<-df %>% 
        e_charts(dummy_x) %>% 
        e_scatter(dummy_y, symbol_size = 10)

    DIRNAME <- normalizePath(dirname(outfile))
    HTMLNAME <- paste( basename(outfile), ".html", sep = "")
    HTMLNAME <- gsub(".html.html$", ".html", HTMLNAME)
    
    if (file.exists(HTMLNAME)) {
        if (overwrite==FALSE) {
            stop("Error: the output HTML file aready exists.")
        } else {
            message("Note: overwriting existing report")
        }
    }

    if (!file.exists(DIRNAME)) {
        stop("Error: the output folder does not exist.")
    }
    
    rmd_tmpdir <- tempdir()
    rmd_tmpfile <- paste(rmd_tmpdir, "/mitch.Rmd", sep = "")
    html_tmp <- paste(paste(rmd_tmpdir, "/mitch_report.html", sep = ""))
    
    DATANAME <- gsub(".html$", ".rds", HTMLNAME)
    DATANAME <- paste(rmd_tmpdir, "/", DATANAME, sep = "")
    saveRDS(res,DATANAME)
    MYMESSAGE <- paste("Dataset saved as \"", DATANAME, "\".")
    message(MYMESSAGE)
    
    knitrenv <- new.env()
    assign("DATANAME", DATANAME, knitrenv)
    assign("res", res, knitrenv)
    
    rmd <- system.file("mitch.Rmd", package = "mitch")
    rmarkdown::render(rmd, intermediates_dir = "." , output_file = html_tmp)
    file.copy(html_tmp, outfile, overwrite=overwrite)
}

map2color <- function(x,pal,limits=NULL){
  if (is.null(limits)) limits <- range(x)
  pal[findInterval(x,seq(limits[1],limits[2],length.out=length(pal)+1), all.inside=TRUE)]
}

gs2net <- function(gset,em,colfunc=colorRampPalette(c("blue", "white","red"))(n=100)){
  gset <- gset[order(names(gset))]
  mydf <- dplyr::bind_rows(lapply(gset, as.data.frame.list))
  rownames(mydf) <- names(gset)
  j <- apply(mydf,1,function(x) {
    apply(mydf,1,function(y) {
      length(intersect(x,y) ) / length(union(x,y))
    })
  })
  j[lower.tri(j)] <- NA
  j[lower.tri(j,diag=TRUE)] <- 0
  jl <- reshape2::melt(j)
  jl <- jl[which(jl$Var1 != jl$Var2),]
  jl <- jl[which(jl$value != 1),]
  jl <- jl[order(-jl$value),]
  jl <- head(jl,length(gset)*2)
  jl$edgeSize = jl$value/sum(jl$value)
  nodes <- sort(union(jl[,1],jl[,2]))
  lengths <- unlist(lapply(gset,length))
  lengths <- lengths[names(lengths) %in% nodes]
  nodes <- data.frame("nodes"=nodes,"lengths"=lengths)
  nodes$vertexsize <- sqrt(nodes$lengths/sum(nodes$lengths) * 100)
  nodes$es <- em[match(nodes$nodes,em$set),"s.dist"]
  nodes$colours <- map2color(nodes$es,colfunc)
  jl2 <- apply(jl[,1:2],2,as.character)
  jlnet <- network(jl2)
  jlnet$val <- lapply(jlnet$val, function(x) {
    vn <- x[[2]]
    vn <- substr(vn, 1, 60)
    if (nchar(vn) == 60 ) {
      vn <- paste(vn,"...",sep="")
    }
  x[[2]] <- vn
  return(x)
  } )
  plot(jlnet, displaylabels = TRUE, label.col = "steelblue",
       edge.lwd = c(jl$edgeSize) * 100,
       arrowhead.cex = 0,
       label.cex = 0.8, vertex.border = "white",vertex.cex = nodes$vertexsize,
       vertex.col = nodes$colours, edge.col = "black")
  E1 <- min(nodes$es)
  E5 <- max(nodes$es)
  E3 <- mean(c(E1,E5))
  EE <- c(E1,E3,E5)
  legcols <- map2color(EE,colfunc)
  legend("topleft", legend=signif(EE,2) ,title="ES",box.lty=0,
    fill=legcols, cex=0.8)
  S1 <- min(nodes$vertexsize)
  FRAG <- S1/10
  S5 <- max(nodes$vertexsize)
  S3 <- mean(c(S1,S5))
  SS <- c(S1-FRAG,0,S5-FRAG)
  L1 <- min(nodes$lengths)
  L5 <- max(nodes$lengths)
  LL <- paste(" ",c(L1,"",L5))
  legend("topright", legend=LL ,title="no. genes",box.lty=0,
    pt.cex=SS*1, cex=0.9 , pch=19,col="darkgray")
  J1 <- min(jl$edgeSize)
  FRAG <- J1*3
  J5 <- max(jl$edgeSize)
  J3 <- mean(c(J1,J5))
  JJ <- c(J1,J3,J5)
  JL <- JJ+FRAG
  JJ <- c( min(jl$value), mean(c(min(jl$value),max(jl$value))),max(jl$value) )
  legend("bottomleft", legend=signif(JJ,2) , lwd=JL*50, title="Jaccard",
    box.lty=0, cex=0.9 , lty=1, col="black")
}

#' networkplot
#'
#' This function generates gene set network diagrams to complement the enrichment analysis.
#' It produces separate charts for up and down-regulated sets.
#' This only includes genes ranked in the top and bottom tertiles.
#' Genes in the set which do not meet this criterion are discarded.
#' The intensity of the colour is proportional to the s.dist value (enrichment score).
#' Circle size is proportional to the number of genes in the set.
#' Line thickness is proportional to the Jaccard similarity value.
#' This function works best after prioritisation with "effect" when running
#' mitch_calc().
#' Note that the circle size and the line width shown in the legend is approximate,
#' although the values shown are exactly the smallest and largest respectively.
#' Note that this chart works best when the width is double the height, otherwise
#' many of the long gene set names could be cut off.
#' There is an element of stochasticity with regard to the network projection, so
#' it could be a good idea to repeat it a few times until you get a nice layout.
#' @param eres a mitch results object.
#' @param FDR the significance threshold for inclusion. By default, it is 0.05
#' @param n_sets the number of sets to include. This type of graph can get messy,
#' so the default is 20, but it can be customised to your needs.
#' @return generates network diagrams.
#' @keywords mitch visualisation network
#' @export
#' @examples
#' data(resExample)
#' networkplot(resExample)
#' @importFrom reshape2 melt
#' @importFrom dplyr bind_rows
#' @importFrom network network
networkplot <- function(eres,FDR=0.05,n_sets=20) {
  scores <- eres[[1]][,1]
  names(scores) <- rownames(eres[[1]])
  gs <- eres[[2]]
  eres <- eres[[4]]
  up <- head(eres[eres$p.adjustANOVA < FDR & eres$s.dist > 0,],n_sets)
  n_up <- nrow(up)
  if (n_up >= 5) {
    up_gs <- up[,1]
    up_gs <- gs[which(names(gs) %in% up_gs)]
    topgs_up <- lapply(seq(from=1,to=length(up_gs)),function(i) {
      gsname <- names(up_gs)[i]
      genes <- up_gs[[i]]
      gene_scores <- scores[which(names(scores) %in% genes)]
      top_genes <- names(which(gene_scores > quantile(gene_scores,c(2/3))))
      return(top_genes)
    })
    names(topgs_up) <- names(up_gs)
    gs2net(gset=topgs_up,em=eres,colfunc=colorRampPalette(c("pink","darkred"))(n=100))
  } else {
    message("Can't plot upregulated sets. Fewer than 5 found.")
  }

  dn <- head(eres[eres$p.adjustANOVA < FDR & eres$s.dist < 0,],n_sets)
  n_dn <- nrow(dn)
  if (n_dn >= 5) {
    dn_gs <- dn[,1]
    dn_gs <- gs[which(names(gs) %in% dn_gs)]
    topgs_dn <- lapply(seq(from=1,to=length(dn_gs)),function(i) {
      gsname <- names(dn_gs)[i]
      genes <- dn_gs[[i]]
      gene_scores <- scores[which(names(scores) %in% genes)]
      top_genes <- names(which(gene_scores < quantile(gene_scores,c(1/3))))
      return(top_genes)
    })
    names(topgs_dn) <- names(dn_gs)
    gs2net(gset=topgs_dn,em=eres,colfunc=colorRampPalette(c("darkblue","lightblue"))(n=100))
  } else {
    message("Can't plot downregulated sets. Fewer than 5 found.")
  }
}

#' network_genes
#'
#' This is a companion function for the network diagrams to display the genes which contribute
#' to the enrichment and are shared between gene sets.
#' @param eres a mitch results object.
#' @param FDR the significance threshold for inclusion. By default, it is 0.05
#' @param n_sets the number of sets to include.
#' @return a list of up- and down-regulated gene sets
#' @keywords mitch visualisation network
#' @export
#' @examples
#' data(resExample)
#' network_genes(resExample)
network_genes <- function(eres,FDR=0.05,n_sets=20) {
  scores <- eres[[1]][,1]
  names(scores) <- rownames(eres[[1]])
  gs <- eres[[2]]
  eres <- eres[[4]]

  up <- head(eres[eres$p.adjustANOVA < FDR & eres$s.dist > 0,],n_sets)
  n_up <- nrow(up)
  if (n_up > 0) {
    up_gs <- up[,1]
    up_gs <- gs[which(names(gs) %in% up_gs)]
    topgs_up <- lapply(seq(from=1,to=length(up_gs)),function(i) {
      gsname <- names(up_gs)[i]
      genes <- up_gs[[i]]
      gene_scores <- scores[which(names(scores) %in% genes)]
      top_genes <- names(which(gene_scores>2))
      return(top_genes)
    })
    names(topgs_up) <- names(up_gs)
  } else {
    topgs_up <- NULL
    message("No significant upregulated sets to show.")
  }

  dn <- head(eres[eres$p.adjustANOVA < FDR & eres$s.dist < 0,],n_sets)
  n_dn <- nrow(dn)
  if (n_dn > 0) {
    dn_gs <- dn[,1]
    dn_gs <- gs[which(names(gs) %in% dn_gs)]
    topgs_dn <- lapply(seq(from=1,to=length(dn_gs)),function(i) {
      gsname <- names(dn_gs)[i]
      genes <- dn_gs[[i]]
      gene_scores <- scores[which(names(scores) %in% genes)]
      top_genes <- names(which(gene_scores<2))
      return(top_genes)
    })
    names(topgs_dn) <- names(dn_gs)
  } else {
    topgs_dn <- NULL 
    message("No significant upregulated sets to show.")
  }
  return(list(c("UP genesets"=topgs_up,"DOWN genesets"=topgs_dn)))
}

#' Reactome gene sets
#'
#' Genesets from Reactome database suitable for enrichment analysis.
#' Acquired August 2019. The structure of this data is a named list of
#' vectors, containing human gene names (character strings). This is 
#' a sample of 200 gene sets from the approximately 2000 present in the
#' full dataset.
#' @docType data
#' @usage data(genesetsExample)
#' @format A list of gene sets
#' @keywords datasets
#' @references Fabregat et al. (2017) BMC Bioinformatics volume 18, 
#' Article number: 142, https://www.ncbi.nlm.nih.gov/pubmed/28249561
#' @source Reactome website: https://reactome.org/
#' @examples
#' data(genesetsExample)
"genesetsExample"

#' H3K36ac profile
#' 
#' Example edgeR result of differential ChIP-seq H3K36ac.
#' This is a dataframe which contains columns for log fold change, log counts
#' per million, p-value and FDR adjusted p-value. These columns consist of 
#' numerical values. The row names represent human gene names. This is a sample
#' of 1000 gene of an original dataset that contains measurements of ~30000 
#' genes.
#' @docType data
#' @usage data(k36a)
#' @format data frame
#' @keywords datasets
#' @examples
#' data(k36a)
"k36a"

#' H3K9ac profile
#' 
#' Example edgeR result of differential ChIP-seq H3K9ac.
#' This is a dataframe which contains columns for log fold change, log counts
#' per million, p-value and FDR adjusted p-value. These columns consist of 
#' numerical values. The row names represent human gene names. This is a sample
#' of 1000 gene of an original dataset that contains measurements of ~30000 
#' genes.
#' @docType data
#' @usage data(k9a)
#' @format data frame     
#' @keywords datasets
#' @examples
#' data(k9a)
"k9a"

#' RNA profile
#' 
#' Example edgeR result of differential RNA expression.
#' This is a dataframe which contains columns for log fold change, log counts
#' per million, p-value and FDR adjusted p-value. These columns consist of 
#' numerical values. The row names represent human gene names. This is a sample
#' of 1000 gene of an original dataset that contains measurements of ~15000 
#' genes.
#' @docType data
#' @usage data(rna)
#' @format data frame     
#' @keywords datasets
#' @examples
#' data(rna)
"rna"

#' myList: A list of three edgeR results 
#' 
#' Example edgeR results of differential RNA, H3K9ac and H3K36ac profiling.
#' The structure of this data is a list of three dataframes. Each data frame
#' is 1000 lines only.
#' @docType data
#' @usage data(myList)
#' @format data frame     
#' @keywords datasets
#' @examples
#' data(myList)
"myList"

#' myImportedData: Example imported profiles
#' 
#' Example of three edgeR profiles imported using mitch.
#' The structure of this data is a dataframe where each column represents
#' one of the following profiling datasets after scoring: RNA, H3K9ac and
#' H3K36ac. Each row represents one gene and this dataset contains just 1000
#' rows to keep the example dataset small.
#' @docType data
#' @usage data(myImportedData)
#' @format data frame     
#' @keywords datasets
#' @examples
#' data(myImportedData)
"myImportedData"

#' resExample: Example mitch result
#' 
#' Example of mitch results. Enrichment of the Reactome gene sets in the RNA,
#' H3K9ac and H3K36ac datasets. The structure of this data set is a list where
#' the 1st element is "input_profile" that has been imported (data frame), 2nd
#' element is the "input_genesets" (names list of gene names [character 
#' vectors]), 3rd is "ranked_profile" which is the input profiling data after
#' ranking (data frame), 4th is "enrichment_result" which is a data frame which
#' provides enrichment information on each gene set in the profiling data
#' including s scores, p-values and FDR adjusted p-values. 5th is 
#' "analysis_metrics" (list). The 6th slot is "detailed_sets" which is a list
#' of 5 matrices which details the enrichment of members of selected gene sets.
#' @docType data
#' @usage data(resExample)
#' @format list of mixed data types
#' @keywords datasets
#' @examples
#' data(resExample)
"resExample"

library("mitch")
library("testthat")

test_that("multiplication works", {
    expect_equal(2 * 2, 4)
})

# 1d
data(rna,genesetsExample)
y<-mitch_import(rna,DEtype="edgeR")
res<-mitch_calc(y,genesetsExample,cores=2)
mitch_plots(res,outfile="1d.pdf") 
if (file.exists("1d.html")) { unlink("1d.html") } 
mitch_report(res,"1d.html")

test_that("1d works", {
    expect_equal(  length(which(res$enrichment_result$p.adjustANOVA<0.1)) ,1)
    expect_true(file.info("1d.pdf")$size>10000)
    expect_true(file.info("1d.html")$size>1000000)
})

unlink("1d.html")
unlink("1d.pdf")


# 1d part 2 to make sure that saving files at a different location works
MYPATH=paste(getwd(),"/1d.html",sep="")
if (file.exists("1d.html")) { unlink("1d.html") }
mitch_report(res,MYPATH)

test_that("1d works", {
    expect_true(file.info("1d.html")$size>1000000)
})

unlink("1d.html")



# 2d
data(rna,k9a,genesetsExample)
x<-list("rna"=rna,"k9a"=k9a)
y<-mitch_import(x,DEtype="edgeR")
res<-mitch_calc(y,genesetsExample,cores=2)
mitch_plots(res,outfile="2d.pdf")
if (file.exists("2d.html")) { unlink("2d.html") }
mitch_report(res,"2d.html")

test_that("2d works", {
    expect_equal(  length(which(res$enrichment_result$p.adjustMANOVA<0.1)) ,1)
    expect_true(file.info("2d.pdf")$size>100000)
    expect_true(file.info("2d.html")$size>1000000)
})

unlink("2d.html")
unlink("2d.pdf")

# 3d
data(rna,k9a,k36a,genesetsExample)
x<-list("rna"=rna,"k9a"=k9a,"k36a"=k36a)
y<-mitch_import(x,DEtype="edgeR")
res<-mitch_calc(y,genesetsExample,cores=2)
mitch_plots(res,outfile="3d.pdf")
if (file.exists("3d.html")) { unlink("3d.html") }
mitch_report(res,"3d.html")

test_that("3d works", {
    expect_equal(  length(which(res$enrichment_result$p.adjustMANOVA<0.1)) ,1)
    expect_true(file.info("3d.pdf")$size>100000)
    expect_true(file.info("3d.html")$size>1000000)
})

unlink("3d.html")
unlink("3d.pdf")

---
title: "Mitch Report"
author: "Antony Kaspi & Mark Ziemann"
output:
  html_document:
    smart: false
theme: cosmo
---

date generated: `r Sys.Date()`

## Background

Mitch performs unidimensional and multidimensional gene set enrichment analysis. The
concept behind this dates to work by Cox and Mann
(https://doi.org/10.1186/1471-2105-13-S16-S12). This implementation is suited to R based
workflows of multi-omics datasets. This software was developed by Antony Kaspi and Mark
Ziemann. Learn more about Mitch at the website: https://github.com/markziemann/Mitch

## Input profiles

Here is the first few lines of the input profile.

```{r,checklibraries,echo=FALSE}

suppressPackageStartupMessages({
    library("mitch")
    library("gtools")
})

```


```{r,peek,echo=FALSE}

res <- readRDS(DATANAME)

# capture the dimensionality of the data                                               
d=ncol(res$input_profile)

# if working with >5 dimensions, then substitute the dimension (colnames) names with a number
if ( d>5 ) {
    mydims<-data.frame(seq_len(d))
    mydims$colnames<-attributes(res)$profile_dimensions
    colnames(mydims)<-c("dimension","contrast_name")
    print(kbl( mydims, caption = "Profile dimensions" ) %>%
    kable_styling("hover", full_width = FALSE))
    colnames(res$input_profile)<- paste("d",seq_len(d),sep="")
    colnames(res$ranked_profile)<- paste("d",seq_len(d),sep="")
    ss<-res$ranked_profile
}

head(res$input_profile) %>%
  kbl(caption="The profiling data being passed to mitch") %>%
  kable_styling("hover", full_width = FALSE)

```

Here are some metrics about the input data profile:

```{r, metrics, echo=FALSE}

if (d==1) {
    formatted<-t(as.data.frame(res$analysis_metrics[ c(1,2,3,4,5 ) ]))
} else if (d==2) {
    unformatted<-t(as.data.frame(res$analysis_metrics[ c(2,3,4,5,11,12 )  ]))
    formatted<-unformatted
    formatted[1:4]<-as.character(round(as.numeric(unformatted[1:4]) , digits=0))
    formatted[5:6]<-as.character(round(as.numeric(unformatted[5:6]) , digits=5))
} else if (d>2) {
    formatted<-t(as.data.frame(res$analysis_metrics[ c(2,3,4,5 )  ]))
}

colnames(formatted)="Profile metrics"

formatted %>%
  kbl(caption="Profiling data metrics") %>%
  kable_styling("hover", full_width = FALSE)

```

Here is a plot of the input profiles. Note the dynamic ranges.

```{r, scatterplot, echo=FALSE,fig.height = 6, fig.width = 6.5, message=FALSE, warning=FALSE}

if ( d==1 ) {
    par(mfrow=c(2,1))
    hist(res$input_profile[,1],breaks=50,main="Distribution of DE scores",xlab=paste("DE score for ",colnames(res$input_profile)))
    plot(res$input_profile[,1],xlab=paste("DE score for ",colnames(res$input_profile)),
    pch="|",frame.plot=FALSE)
    UPS=length(which(res$input_profile[,1]>0))
    DNS=length(which(res$input_profile[,1]<0))
    TOTAL=nrow(res$input_profile)
    mtext(paste(TOTAL,"genes in total,",UPS,"trending up-regulated,",DNS,"trending down-regulated"))
} else if ( d<3 ) {
    plot(res$input_profile, pch=19, col=rgb(red = 0, green = 0, blue = 0, alpha = 0.15), main="Input profiles"  )
} else {
    ggpairs_points_plot <- function(data ,mapping, ...){
        p <- ggplot(data = data, mapping = mapping) +
        geom_point(alpha=0.05) +
        geom_vline(xintercept=0,linetype="dashed") +
        geom_hline(yintercept=0,linetype="dashed")
    }
    p<-ggpairs(as.data.frame(res$input_profile), title="Scatterplot of all genes" , lower  = list(continuous = ggpairs_points_plot ))
    print( p +  theme_bw() )
}

```

Here is the contour plot of the profile including all detected genes.

```{r, contourplot, echo=FALSE,fig.height = 6, fig.width = 6.5,warning=FALSE, message=FALSE}

palette <- colorRampPalette(c("white", "yellow","orange" ,"red","darkred","black"))

#Contour of all the data
ss<-res$ranked_profile

if (d==1) {
    message("Contour plot does not apply to unidimensional analysis.")
} else if (d==2) {
    xmin=min(ss[,1])
    xmax=max(ss[,1])
    ymin=min(ss[,2])
    ymax=max(ss[,2])
    ss<-res$ranked_profile
    k<-MASS:::kde2d(ss[,1],ss[,2])
    X_AXIS=paste("Rank in contrast",colnames(ss)[1])
    Y_AXIS=paste("Rank in contrast",colnames(ss)[2])
    filled.contour(k, xlim=c(xmin,xmax),ylim=c(ymin,ymax),
    color=palette ,
    plot.title={ abline(v=0,h=0,lty=2,lwd=2,col="blue")
    title( main="Rank-rank plot of all genes",xlab=X_AXIS,ylab=Y_AXIS ) } )
} else if (d>2) {
    #pairs contour plot function
    ggpairs_func <- function(data, mapping, ...){
        p <- ggplot(data = data, mapping = mapping) +
        stat_density2d(aes(fill=..density..), geom="tile", contour = FALSE) +
        geom_vline(xintercept=0,linetype="dashed") +
        geom_hline(yintercept=0,linetype="dashed") +
        scale_fill_gradientn(colours=palette(25))

        p

    }

    #pairs contour plot
    p<-ggpairs(as.data.frame(ss), title="Contour plot of all genes after ranking" ,
    lower=list(continuous=ggpairs_func),
    diag=list(continuous=wrap("barDiag", binwidth=nrow(ss)/100)))

    print( p + theme_bw() )

    #subset contour plot
    ggpairs_contour_limit_range <- function(data ,mapping, ...){

        p <- ggplot(data = data, mapping = mapping) +
        stat_density2d(aes(fill=..density..), geom="tile", contour = FALSE) +
        geom_vline(xintercept=0,linetype="dashed") +
        geom_hline(yintercept=0,linetype="dashed") +
        scale_fill_gradientn(colours=palette(25)) +
        scale_x_continuous( limits = range(min(ss[,gsub("~","",as.character(mapping[1]))]),max(ss[,gsub("~","",as.character(mapping[1]))])) ) +
        scale_y_continuous( limits = range(min(ss[,gsub("~","",as.character(mapping[2]))]),max(ss[,gsub("~","",as.character(mapping[2]))])) )

        p

    }

    #subset points plot
    ggpairs_points_limit_range <- function(data ,mapping, ...){

        p <- ggplot(data = data, mapping = mapping) +
        geom_point(alpha=0.1) +
        geom_vline(xintercept=0,linetype="dashed") +
        geom_hline(yintercept=0,linetype="dashed") +
        scale_x_continuous( limits = range(min(ss[,gsub("~","",as.character(mapping[1]))]),max(ss[,gsub("~","",as.character(mapping[1]))])) ) +
        scale_y_continuous( limits = range(min(ss[,gsub("~","",as.character(mapping[2]))]),max(ss[,gsub("~","",as.character(mapping[2]))])) )

        p

    }

}

```

## Input genesets

Here are some metrics about the gene sets used:

```{r,input_geneset_metrics1,results="asis",echo=FALSE}

ORIGINFILE=attributes(res$input_genesets)$originfile
cat(paste("GMT file of genesets:",ORIGINFILE,"<br>"))
unformatted<-t(as.data.frame(res$analysis_metrics[c(1,6,7)]))
formatted<-as.data.frame(as.character( unformatted[1:3]) )
rownames(formatted)=rownames(unformatted)
colnames(formatted)="Gene sets metrics"

formatted %>%
  kbl(caption="Gene set metrics") %>%
  kable_styling("hover", full_width = F)

```

```{r,input_geneset_metrics2,results="asis",echo=FALSE ,fig.height = 7, fig.width = 7 ,fig.show="all"}

par(mfrow=c(3,1))
geneset_counts<-res$analysis_metrics$geneset_counts
boxplot(geneset_counts$count,horizontal=TRUE,frame=FALSE,main="Gene set size",xlab="number of member genes included in profile")
hist(geneset_counts$count,100,xlab="geneset size",main="Histogram of geneset size")
hist(geneset_counts$count,100,xlim=c(0,500),xlab="geneset size",main="Trimmed histogram of geneset size")

if ( d==2 ) {
    uu=length(which(res$input_profile[,1]>0 & res$input_profile[,2]>0))
    ud=length(which(res$input_profile[,1]>0 & res$input_profile[,2]<0))
    dd=length(which(res$input_profile[,1]<0 & res$input_profile[,2]<0))
    du=length(which(res$input_profile[,1]<0 & res$input_profile[,2]>0))
    a<-as.data.frame(c(uu,ud,dd,du))
    rownames(a)=c("top-right","bottom-right","bottom-left","top-left")
    colnames(a)="a"
    par(mfrow=c(1,1))
    xx<-barplot(a$a,names.arg=rownames(a),main="number of genes in each quadrant")
    text(x = xx, y = a$a, label = a$a , pos = 1, cex = 1)
} else if (d>2) {
    if (d<6) {
        sig<-sign(ss)
        sector_count<-aggregate(seq_len(nrow(sig)) ~ ., sig, FUN = length)
        colnames(sector_count)[ncol(sector_count)]<-"Count"

         sector_count %>%
         kbl(caption="Genes by sector") %>%
         kable_styling("hover", full_width = FALSE)
    }

}

```

```{r,input_geneset_metrics3,results="asis",echo=FALSE,message=FALSE,fig.height = 7, fig.width = 7}

if (d<2) {
    nsig=length(which(res$enrichment_result$p.adjustANOVA<0.05))
} else {
    nsig=length(which(res$enrichment_result$p.adjustMANOVA<0.05))
}

if (d==1) {
    cat("<h2>Differential pathway expression</h2><br>")
    par(mfrow=c(1,1))
    sig<-subset(res$enrichment_result,p.adjustANOVA<=0.05)

    plot(res$enrichment_result$s.dist,-log10(res$enrichment_result$pANOVA),
      xlab="s score",ylab="-log10(p-value)",
      main="volcano plot of gene set enrichments",pch=19,cex=0.8)

    points(sig$s.dist,-log10(sig$pANOVA),pch=19,cex=0.85,col="red")
    TOTAL=nrow(res$enrichment_result)
    SIG=nrow(sig)
    UP=length(which(sig$s.dist>0))
    DN=length(which(sig$s.dist<0))
    SUBHEADER=paste(TOTAL,"gene sets in total,",UP,"upregulated and ",DN,"downregulated (FDR<=0.05)")
    mtext(SUBHEADER)
}

if (d==2) {
    cat("<h2>Gene sets by quadrant</h2><br>")
    cat(paste("Number of significant gene sets (FDR<0.05)=", res$analysis_metrics$num_sets_significant,"<br>" ))
    a<-res$analysis_metrics[14]
    a<-as.data.frame(as.numeric(unlist(strsplit(as.character(a),','))),stringsAsFactors=FALSE)
    rownames(a)=c("top-right","bottom-right","bottom-left","top-left")
    colnames(a)="a"
    xx<-barplot(a$a,names.arg=rownames(a),main="number of genesets FDR<0.05")
    text(x = xx, y = a$a, label = a$a , pos = 1, cex = 1)
}

if ( nsig > 0 ) {
    if ( d>2 ) {
        if ( d<6 ) {
            cat(paste("Number of significant gene sets (FDR<0.05)=", res$analysis_metrics$num_sets_significant,"<br>" ))
            cat("<h2>Gene sets by sector</h2><br>")
            sig<-sign(res$enrichment_result[which(res$enrichment_result$p.adjustMANOVA<0.05),4:(4+d-1)])
            sector_count<-aggregate(seq_len(nrow(sig)) ~ ., sig, FUN = length)
            colnames(sector_count)[ncol(sector_count)]<-"Count"

            sector_count %>%
            kbl(caption="Gene sets by sector") %>%
            kable_styling("hover", full_width = FALSE)
        }
    }
}

```

## Interactive enrichment scatterplot

```{r,echart1d,results="asis",echo=FALSE, fig.height = 7, fig.width = 7 ,fig.show="all", message=FALSE}

if (d==1) {
    numsets=nrow(subset(res$enrichment_result,p.adjustANOVA<0.05))
    p=NULL
    if (numsets==0){
        message("No significant enrichments found.")
    } else {
        # volcano with echarts4r
        cat("Significance is calculated by -log10(p-value). All points shown are FDR<0.05.<br>")
        myres2<-subset(res$enrichment_result,p.adjustANOVA<0.05)
        myres2$significance<--log10(myres2$pANOVA)
        myres2$set<-sub(",","",myres2$set)
        XCOL=colnames(myres2)[4]
        YCOL=colnames(myres2)[6]
        colnames(myres2)[4]<-"xx"
        colnames(myres2)[6]<-"yy"

        p2<-myres2 %>%
            tibble::rownames_to_column("model") %>%
            dplyr::mutate(set = paste(set, setSize, p.adjustANOVA , sep = ",")) %>%
            e_charts(x = xx) %>%
            e_legend(show = FALSE) %>%
            e_x_axis(name = XCOL) %>%  # add x axis name
            e_y_axis(name = YCOL) %>%  # add y axis name
            e_scatter( serie = yy , bind = set , symbolSize = 10  ) %>%
            e_tooltip(formatter = htmlwidgets::JS("
                function(params){
                var vals = params.name.split(',')
                return('<strong>' + vals[0] +
                '</strong><br />s.x-axis: ' + parseFloat(params.value[0]).toFixed(2) +
                '<br />s.y-axis: ' +  parseFloat(params.value[1]).toFixed(2)) +
                '<br />setSize: ' + vals[1] +
                '<br />p-adjust ANOVA: ' + Number(vals[2]).toPrecision(2)
            }"))
    p2
    }
}

if (d==1) {
    resrows=length(res$detailed_sets)
    if (resrows>1) {
        myres2<-head(res$enrichment_result,resrows)
        myres2$significance<--log10(myres2$pANOVA)
        myres2$set<-sub(",","",myres2$set)
        XCOL=colnames(myres2)[4]
        YCOL=colnames(myres2)[6]
        colnames(myres2)[4]<-"xx"
        colnames(myres2)[6]<-"yy"

        p2<-myres2 %>%
            tibble::rownames_to_column("model") %>%
            mutate(set = paste(set, setSize, p.adjustANOVA , sep = ",")) %>%
            e_charts(x = xx) %>%
            e_legend(show = FALSE) %>%
            e_x_axis(name = XCOL) %>%  # add x axis name
            e_y_axis(name = YCOL) %>%  # add y axis name
            e_scatter( serie = yy , bind = set , symbolSize = 10  ) %>%
            e_tooltip(formatter = htmlwidgets::JS("
                function(params){
                var vals = params.name.split(',')
                return('<strong>' + vals[0] +
                '</strong><br />s.x-axis: ' + parseFloat(params.value[0]).toFixed(2) +
                '<br />s.y-axis: ' +  parseFloat(params.value[1]).toFixed(2)) +
                '<br />setSize: ' + vals[1] +
                '<br />p-adjust ANOVA: ' + Number(vals[2]).toPrecision(2)
            }"))

        cat("Significance is calculated by -log10(p-value). Top N sets shown irrespective of FDR.<br>")
    p2
    }
}

```{r,echart2d,results="asis",echo=FALSE, fig.height = 7, fig.width = 7 ,fig.show="all", message=FALSE}

echartscatter<-function(i){
    my_x=plan[i,1]
    my_y=plan[i,2]
    XCOL=colnames(myres)[3+my_x]
    YCOL=colnames(myres)[3+my_y]
    myres$set<-sub(","," ",as.character(myres$set))
    colnames(myres)[3+my_x]<-"xx"
    colnames(myres)[3+my_y]<-"yy"

    p2 <- myres %>%
        tibble::rownames_to_column("model") %>%
        mutate(set = paste(set, setSize, p.adjustMANOVA , s.dist , sep = ",")) %>%
        e_charts(x = xx , height = 300 ) %>%
        e_legend(show = FALSE) %>%
        e_x_axis(name = XCOL) %>%  # add x axis name
        e_y_axis(name = YCOL) %>%  # add y axis name
        e_scatter( serie = yy , bind = set , symbolSize = 10  ) %>%
        e_tooltip(formatter = htmlwidgets::JS("
            function(params){
            var vals = params.name.split(',')
            return('<strong>' + vals[0] +
            '</strong><br />s.x-axis: ' + parseFloat(params.value[0]).toFixed(2) +
            '<br />s.y-axis: ' +  parseFloat(params.value[1]).toFixed(2)) +
            '<br />setSize: ' + vals[1] +
            '<br />p-adjust MANOVA: ' + Number(vals[2]).toPrecision(2) +
            '<br />s.dist: ' + Number(vals[3]).toPrecision(2)
        }"))
    p2
}

if (d!=1) {
    cat("All sets with FDR<0.05. Try hovering over the points.<br>")

    numsets=nrow(subset(res$enrichment_result,p.adjustMANOVA<0.05))

    p=NULL

    if (numsets<1){

        message("No significant enrichments found.")
        p2<-list()

    } else {

        myres<-subset(res$enrichment_result,p.adjustMANOVA<0.05)

        if ( d<3 ) {

            myres2<-myres[,c(1,which(names(myres) %in% "p.adjustMANOVA"),2, 4:(4+d-1) , which(names(myres) %in% "s.dist") )]
            myres2$set<-sub(",","",myres2$set)
            XCOL=colnames(myres2)[4]
            YCOL=colnames(myres2)[5]
            colnames(myres2)[4]<-"xx"
            colnames(myres2)[5]<-"yy"

            p2 <- myres2 %>%
                tibble::rownames_to_column("model") %>%
                mutate(set = paste(set, setSize, p.adjustMANOVA , s.dist , sep = ",")) %>%
                e_charts(x = xx) %>%
                e_legend(show = FALSE) %>%
                e_x_axis(name = XCOL) %>%  # add x axis name
                e_y_axis(name = YCOL) %>%  # add y axis name
                e_scatter( serie = yy , bind = set , symbolSize = 10  ) %>%
                e_tooltip(formatter = htmlwidgets::JS("
                    function(params){
                    var vals = params.name.split(',')
                    return('<strong>' + vals[0] +
                    '</strong><br />s.x-axis: ' + parseFloat(params.value[0]).toFixed(2) +
                    '<br />s.y-axis: ' +  parseFloat(params.value[1]).toFixed(2)) +
                    '<br />setSize: ' + vals[1] +
                    '<br />p-adjust MANOVA: ' + Number(vals[2]).toPrecision(2) +
                    '<br />s.dist: ' + Number(vals[3]).toPrecision(2)
                }"))
        } else {
            plan<-combinations(n = d, r = 2, v = seq_len(d), repeats.allowed = FALSE)
            myres<-myres[,c(1, which(names(myres) %in% "p.adjustMANOVA") ,2,4:(4+d-1),
                which(names(myres) %in% "s.dist") ), which(names(myres) %in% "SD")]
            p2<-list()
            p2<-lapply( seq_len(nrow(plan)) , echartscatter)
        }
    htmltools::tagList(p2)
    }
}

if (d!=1) {
    cat("Top N sets irrespective of FDR. Try hovering over the points.<br>")
    resrows=length(res$detailed_sets)
    p=NULL

    if (resrows<1){
        message("No results found.")
    } else {
        myres<-head(res$enrichment_result,resrows)

        if ( d<3 ) {
            myres2<-myres[,c(1,which(names(myres) %in% "p.adjustMANOVA"),2, 4:(4+d-1) ,
                which(names(myres) %in% "s.dist") )]

            myres2$set<-sub(",","",myres2$set)
            XCOL=colnames(myres2)[4]
            YCOL=colnames(myres2)[5]
            colnames(myres2)[4]<-"xx"
            colnames(myres2)[5]<-"yy"

            p2<-myres2 %>%
                tibble::rownames_to_column("model") %>%
                mutate(set = paste(set, setSize, p.adjustMANOVA , s.dist , sep = ",")) %>%
                e_charts(x = xx) %>%
                e_legend(show = FALSE) %>%
                e_x_axis(name = XCOL) %>%  # add x axis name
                e_y_axis(name = YCOL) %>%  # add y axis name
                e_scatter( serie = yy , bind = set , symbolSize = 10  ) %>%
                e_tooltip(formatter = htmlwidgets::JS("
                    function(params){
                    var vals = params.name.split(',')
                    return('<strong>' + vals[0] +
                    '</strong><br />s.x-axis: ' + parseFloat(params.value[0]).toFixed(2) +
                    '<br />s.y-axis: ' +  parseFloat(params.value[1]).toFixed(2)) +
                    '<br />setSize: ' + vals[1] +
                    '<br />p-adjust MANOVA: ' + Number(vals[2]).toPrecision(2) +
                    '<br />s.dist: ' + Number(vals[3]).toPrecision(2)
                }"))
        } else {
            plan<-combinations(n = d, r = 2, v = seq_len(d), repeats.allowed = FALSE)

            myres<-myres[,c(1, which(names(myres) %in% "p.adjustMANOVA") ,2,4:(4+d-1),
                which(names(myres) %in% "s.dist") ), which(names(myres) %in% "SD")]

            p<-list()
            lapply( seq_len(nrow(plan)) , echartscatter)
        }
    htmltools::tagList(p)
    htmltools::tagList(p2)
    }
}

```

```{r,heatmap,results="asis",echo=FALSE, fig.height = 10, fig.width = 7 ,fig.show="all", message=FALSE}

if (d!=1) {
    if (resrows>2) {
        cat("<h2> A heatmap of S values for top results</h2><br>")
        hmapx<-head( res$enrichment_result[,4:(4+d-1)] ,resrows)
        rownames(hmapx)<-head(res$enrichment_result$set,resrows)
        colnames(hmapx)<-gsub("^s.","",colnames(hmapx))
        my_palette <- colorRampPalette(c("blue", "white", "red"))(n = 25)

        if(resrows<25){
            CEXROW=1
        } else if (resrows<51) {
            CEXROW=0.8
        } else if (resrows<76) {
            CEXROW=0.6
        } else if (resrows<101) {
            CEXROW=0.4
        } else {
            CEXROW=0.25
        }
        heatmap.2(as.matrix(hmapx),scale="none",margin=c(10, 25),cexRow=CEXROW,trace="none",cexCol=1,col=my_palette)
    }
}

```

```{r,effectsize,results="asis",echo=FALSE, fig.height = 7, fig.width = 7 ,fig.show="all", message=FALSE}

if (d!=1) {
    cat('<h2> A plot of effect size versus significance</h2><br>')
    cat('Significance is the -log2(p.adjustMANOVA) and effect size is the s.dist which is the hypotenuse of the s scores.<br>')
    colnames(myres)<-gsub("\\.","_",colnames(myres))
    myres<-res$enrichment_result
    myres$significance<--log2(myres$p.adjustMANOVA)
    myres$set<-sub(",","",myres$set)
    XCOL=colnames(myres)[ncol(myres)-3]
    YCOL=colnames(myres)[ncol(myres)]
    colnames(myres)[ncol(myres)-3]<-"xx"
    colnames(myres)[ncol(myres)]<-"yy"

    p2<-myres %>%
        tibble::rownames_to_column("model") %>%
        mutate(set = paste( set , setSize , p.adjustMANOVA , sep = ",")) %>%
        e_charts( x = xx) %>%
        e_legend(show = FALSE) %>%
        e_x_axis(name = XCOL) %>%  # add x axis name
        e_y_axis(name = YCOL) %>%  # add y axis name
        e_scatter( serie = yy , bind = set , symbolSize = 10  ) %>%
        e_tooltip(formatter = htmlwidgets::JS("
            function(params){
            var vals = params.name.split(',')
            return('<strong>' + vals[0] +
            '</strong><br />s.x-axis: ' + parseFloat(params.value[0]).toFixed(2) +
            '<br />s.y-axis: ' +  parseFloat(params.value[1]).toFixed(2)) +
            '<br />setSize: ' + vals[1] +
            '<br />p-adjust MANOVA: ' + Number(vals[2]).toPrecision(2)
        }"))
    p2
}

```

## Results table

```{r,results_table,results="asis",echo=FALSE}

if ( d==1 ) {
    resrows=length(res$detailed_sets)
    myres<-head(res$enrichment_result,resrows)
    myres[,c(3:ncol(myres))]<-signif(myres[,c(3:ncol(myres))],3)
} else if (d>1) {
    resrows=length(res$detailed_sets)
    myres<-head(res$enrichment_result,resrows)
    myres<-myres[,c(1:3, which(names(myres) %in% "p.adjustMANOVA"), which(names(myres) %in% "s.dist") ,4:((d*2)+3)), which(names(myres) %in% "SD")]
    myres[,3:ncol(myres)]<-signif(myres[,3:ncol(myres)],3)
}

formatted<-myres
formatted[,1]<-gsub("_"," ",myres[,1])

kbl( formatted , format="markdown", row.names=FALSE, caption = cat(paste("Top N=",resrows,"gene sets")) ,digits=100) %>%
kable_styling("hover", full_width = FALSE)

cat("<hr><br>")

```

## Results (complete table)

```{r,results_table_complete,results="asis",echo=FALSE}

myres<-res$enrichment_result

if ( d==1 ) {
    myres[,c(3:ncol(myres))]<-signif(myres[,c(3:ncol(myres))],3)
    formatted<-myres
    formatted[,1]<-gsub("_"," ",myres[,1])
} else if (d>1) {
    myres<-myres[,c(1:3, which(names(myres) %in% "p.adjustMANOVA"), which(names(myres) %in% "s.dist") ,4:((d*2)+3)), which(names(myres) %in% "SD")]
    myres[,3:ncol(myres)]<-signif(myres[,3:ncol(myres)],3)
    formatted<-myres
    formatted[,1]<-gsub("_"," ",myres[,1])
    formatted[,3:ncol(myres)]<-format(myres[,3:ncol(myres)],digits=3)
}

HEADER=paste("<br><details><summary><b>","Click HERE to show results for all gene sets","</b></summary><br><p>",sep=" " )

cat(HEADER)

kbl(formatted, format="html", row.names=FALSE, caption = "Complete results",digits=100) %>%
kable_styling("hover", full_width = FALSE)

cat("<br></p></details>")
cat("<hr><br>")

```

```{r,detailed_geneset_reports1d,results="asis",echo=FALSE,fig.height=6, fig.width=6 ,out.width = '80%',comment=NA, message=FALSE}

if (d==1) {
    cat("<h2>Detailed Gene set reports</h2><br>")
    cat('<br>')
    cat('\n')
    ss<-res$ranked_profile

    for ( i in seq_len(resrows) ) {
        mydat = NULL
        mydat<-t(myres[i,])

        cat(paste("<b>",as.character(myres[i,1]) ,"</b><br>"))

        tbl <- kbl(mydat,col.names = NULL,format='pipe',caption=NULL ) %>%
            kable_styling("hover", full_width = FALSE)

        cat(tbl)
        cat('\n\n<!-- -->\n\n')
        cat("<br>")

        # plots
        sss<-res$detailed_sets[[i]]
        set<-names(res$detailed_sets[i])
        size<-length(sss)
        par(mfrow=c(3,1))

        beeswarm(sss,vertical = FALSE,cex=0.75,xlim=c(min(ss),
        max(ss)),col="darkgray",pch=19,main=set,cex.main=1.5,
        xlab=paste("ranked DE score in:",colnames(ss)))

        mtext("beeswarm plot",cex=0.8)

        hist(sss,xlim=c(min(ss),max(ss)),breaks=15,col="darkgray",main=NULL,
        border="black",xlab=paste("ranked DE score in:",colnames(ss)))

        mtext("histogram",cex=0.8)

        plot(sss,rep(1,length(sss)),type="n",xlim=c(min(ss),max(ss)),
        frame=FALSE,axes=FALSE,ylab="",
        xlab=paste("ranked DE score in:",colnames(ss)))

        rug(sss, ticksize = 0.9)
        axis(1)
        mtext("rugplot",cex=0.8)
        cat("<br>")

        # top gene list
        setSign=sign(res$enrichment_result[i,4])

        if (setSign==-1) {
            tops<-as.data.frame(res$detailed_sets[[i]][order(res$detailed_sets[[i]])])
        } else if (setSign==1) {
            tops<-as.data.frame(res$detailed_sets[[i]][order(-res$detailed_sets[[i]])])
        }

        cat("Top enriched genes\\n")
        tops$GeneID<-rownames(tops)
        tops<-tops[,c(2,1)]
        colnames(tops)=c("GeneID","Gene Rank")
        cat("<br>")

        tbl <- kbl(head(tops,n=20L), format='pipe', row.names=FALSE,caption="Top 20 genes",digits=100) %>%
                kable_styling("hover", full_width = FALSE)

        cat(tbl)
        cat('\n\n<!-- -->\n\n')
        HEADER=paste("<br><details><summary><b>","Click HERE to show all gene set members","</b></summary><br><p>",sep=" " )
        cat(HEADER)

        tbl <- kbl(tops, format='pipe', row.names=FALSE, caption="All member genes",digits=100) %>%
                kable_styling("hover", full_width = FALSE)

        cat(tbl)
        cat('\n\n<!-- -->\n\n')
        cat("<br></p></details>")
        cat("<br><hr>")
    }
}

```

```{r,detailed_geneset_reports2d,results="asis",echo=FALSE,fig.height=5, fig.width=6 ,out.width = '80%',comment=NA, message=FALSE}

twodimplot<-function(i) {
    ll<-res$enrichment_result[i,]
    size<-ll$setSize
    sss<-res$detailed_sets[[i]]
    k<-MASS:::kde2d(sss[,1],sss[,2])
    filled.contour( k, color = palette, xlim=c(xmin,xmax),ylim=c(ymin,ymax),
    plot.title={ abline(v=0,h=0,lty=2,lwd=2,col="blue")
    title( main=ll$set , xlab=X_AXIS,ylab=Y_AXIS )})

    plot(sss, pch=19, col=rgb(red = 0, green = 0, blue = 0, alpha = 0.2),
    main=ll$set , xlim=c(xmin,xmax),ylim=c(ymin,ymax),
    xlab=X_AXIS,ylab=Y_AXIS)

    abline(v=0,h=0,lty=2,lwd=2,col="blue")
    sss_long<-melt(sss)
    colnames(sss_long) <- c("gene","contrast","value")
    colnames(ss_long) <- c("gene","contrast","value")

    p<-ggplot(ss_long,aes(contrast,value)) +
    geom_violin(data=ss_long,fill = "grey", colour = "grey") +
    geom_boxplot(data=ss_long,width=0.9,fill="grey",outlier.shape = NA) +
    geom_violin(data=sss_long,fill = "black", colour = "black") +
    geom_boxplot(data=sss_long,width=0.1,outlier.shape = NA) +
    labs(y = "Position in rank",title = ll[,1] )

    print( p + theme_bw() +
    theme( axis.text=element_text(size=14),
    axis.title=element_text(size=15),
    plot.title = element_text(size = 14)))
}


# subset contour plot
ggpairs_contour_limit_range <- function(data ,mapping, ...){
    p <- ggplot(data = data, mapping = mapping) +
    stat_density2d(aes(fill=..density..), geom="tile", contour = FALSE) +
    geom_vline(xintercept=0,linetype="dashed") +
    geom_hline(yintercept=0,linetype="dashed") +
    scale_fill_gradientn(colours=palette(25)) +
    scale_x_continuous( limits = range(min(ss[,gsub("~","",as.character(mapping[1]))]),max(ss[,gsub("~","",as.character(mapping[1]))])) ) +
    scale_y_continuous( limits = range(min(ss[,gsub("~","",as.character(mapping[2]))]),max(ss[,gsub("~","",as.character(mapping[2]))])) )
    p
}


# subset points plot
ggpairs_points_limit_range <- function(data ,mapping, ...){
    p <- ggplot(data = data, mapping = mapping) +
    geom_point(alpha=0.1) +
    geom_vline(xintercept=0,linetype="dashed") +
    geom_hline(yintercept=0,linetype="dashed") +
    scale_x_continuous( limits = range(min(ss[,gsub("~","",as.character(mapping[1]))]),max(ss[,gsub("~","",as.character(mapping[1]))])) ) +
    scale_y_continuous( limits = range(min(ss[,gsub("~","",as.character(mapping[2]))]),max(ss[,gsub("~","",as.character(mapping[2]))])) )
    p
}


ndplot<-function(i) {
    ll<-res$enrichment_result[i,]
    size<-ll$setSize
    ss<-res$ranked_profile
    sss<-res$detailed_sets[[i]]

    if ( d>5 ) {
        colnames(sss)<- paste("d",seq_len(d),sep="")
    }

    empty_cnt <- length(which(is.na(cor(sss,use="pairwise.complete.obs"))))
    if ( empty_cnt == 0 ) {

        p<-ggpairs(as.data.frame(sss), title=ll[,1], lower=list(continuous=ggpairs_contour_limit_range),
        diag=list(continuous=wrap("barDiag", binwidth=nrow(ss)/10)) )

        print( p + theme_bw() )

        p<-ggpairs(as.data.frame(sss), title=ll[,1], lower= list(continuous = ggpairs_points_limit_range ),
        diag=list(continuous=wrap("barDiag", binwidth=nrow(ss)/10)))

        print( p + theme_bw() )
    }

    sss_long<-melt(sss)
    colnames(sss_long) <- c("gene","contrast","value")
    colnames(ss_long) <- c("gene","contrast","value")

    empty_cnt <- apply(sss,2,function(x) sum(as.numeric(is.finite(x))))
    empty_cnt <- length(which(empty_cnt==0))
    if ( empty_cnt == 0 ) {

        p<-ggplot(ss_long,aes(contrast,value)) +
        geom_violin(data=ss_long,fill = "grey", colour = "grey") +
        geom_boxplot(data=ss_long,width=0.9,fill="grey",outlier.shape = NA) +
        geom_violin(data=sss_long,fill = "black", colour = "black") +
        geom_boxplot(data=sss_long,width=0.1,outlier.shape = NA) +
        labs(y = "Position in rank",title = ll[,1] )

        print( p + theme_bw() +
        theme( axis.text=element_text(size=14),
        axis.title=element_text(size=15),
        plot.title = element_text(size = 14)))
    }
}


topgenelist<-function(i) {
    ss<-res$ranked_profile
    sss<-res$detailed_sets[[i]]

    if ( d>5 ) {
        colnames(sss)<- paste("d",seq_len(d),sep="")
    }

    tl=bl=tr=br=0
    myres<-head(res$enrichment_result,resrows)
    myres<-myres[,c(1:3, which(names(myres) %in% "p.adjustMANOVA"), which(names(myres) %in% "s.dist") ,4:((d*2)+3)) , which(names(myres) %in% "s.dist")]
    myres[,3:ncol(myres)]<-signif(myres[,3:ncol(myres)],3)

    #select 2 strongest columns to highlight genes
    if( d>2 ) {
        cols<-(ncol(myres)-(2*d)+1):(ncol(myres)-(2*d)+d)
        COLS<-order(-abs(unlist((myres[i,cols]))))[1:2]
        sss<-sss[,COLS]
        cols<-cols[COLS]
        mysx=signif(myres[i,cols[1]],3)
        mysy=signif(myres[i,cols[2]],3)
    } else {
        cols=c(ncol(myres)-3,ncol(myres)-2)
        mysx=signif(myres[i,cols[1]],3)
        mysy=signif(myres[i,cols[2]],3)
    }

    if ( mysy>0 ) { tl=tl+1 ; tr=tr+1  } else { bl=bl+1 ; br=br+1 }
    if ( mysx>0 ) { tr=tr+1 ; br=br+1  } else { tl=tl+1 ; bl=bl+1 }

    if (bl==2) {
        myquad<-sss[which(sss[,1]<0 & sss[,2]<0),,drop=FALSE]
        topgenes<-myquad[order(-rank(myquad[,1]*myquad[,2])),,drop=FALSE]
    }

    if (tr==2) {
        myquad<-sss[which(sss[,1]>0 & sss[,2]>0),,drop=FALSE]
        topgenes<-myquad[order(-rank(myquad[,1]*myquad[,2])),,drop=FALSE]
    }

    if (br==2) {
        myquad<-sss[which(sss[,1]>0 & sss[,2]<0),,drop=FALSE]
        topgenes<-myquad[order(rank(myquad[,1]*myquad[,2])),,drop=FALSE]
    }

    if (tl==2) {
        myquad<-sss[which(sss[,1]<0 & sss[,2]>0),,drop=FALSE]
        topgenes<-myquad[order(rank(myquad[,1]*myquad[,2])),,drop=FALSE]
    }

    cat("<br>")
    topgenes<-as.data.frame(topgenes)
    topgenes$Gene<-as.character(rownames(topgenes))
    topgenes<-topgenes[,c(3,1,2)]

    tbl <- kbl(head(topgenes,n=20L), format='pipe', row.names=FALSE, caption="Top 20 genes", digits=100) %>%
        kable_styling("hover", full_width = FALSE)

    cat(tbl)
    cat('\n\n<!-- -->\n\n')
    HEADER=paste("<br><details><summary><b>","Click HERE to show all gene set members","</b></summary><br><p>",sep=" " )
    cat(HEADER)
    sss<-res$detailed_sets[[i]]

    tbl <- kbl(sss, format='pipe', row.names=TRUE, caption="All member genes", digits=100) %>%
        kable_styling("hover", full_width = FALSE)

    cat(tbl)
    cat('\n\n<!-- -->\n\n')
    cat("<br></p></details>")
    cat("<br><hr>")
}


#functions end here

if (d!=1) {
    cat("<h2> Detailed Gene set reports</h2><br>")
    resrows=length(res$detailed_sets)
    ss<-res$ranked_profile
    myres<-head(res$enrichment_result,resrows)
    myres<-myres[,c(1:3, which(names(myres) %in% "p.adjustMANOVA"), which(names(myres) %in% "s.dist") ,4:((d*2)+3)), which(names(myres) %in% "SD")]
    myres[,3:ncol(myres)]<-signif(myres[,3:ncol(myres)],3)
    palette <- colorRampPalette(c("white", "yellow","orange" ,"red","darkred","black"))
    ss_long<-melt(ss)
    colnames(ss_long) <- c("gene","contrast","value")

    if ( d<3 ) {
        xmin=min(ss[,1])
        xmax=max(ss[,1])
        ymin=min(ss[,2])
        ymax=max(ss[,2])

            for ( i in seq_len(resrows )) {
                mydat = NULL
                mydat$metrics<-names(myres[2:ncol(myres)])
                mydat$values<-unname(t(myres[i,2:ncol(myres)]))
                mydat<-as.data.frame(cbind(mydat$metrics,mydat$values))
                colnames(mydat)<-c("metric","value")
                cat(paste("<b>",as.character(myres[i,1]) ,"</b><br>"))

                tbl <- kbl(mydat,format='pipe', caption=as.character(t(myres[i,1]))) %>%
                    kable_styling("hover", full_width = FALSE)

                cat(tbl)
                cat('\n\n<!-- -->\n\n')
                cat("<br>")
                twodimplot(i)
                cat("<br>")
                topgenelist(i)
                cat("<hr><br>")
            }
        } else {

        for ( i in seq_len(resrows )) {
            mydat = NULL
            mydat$metrics<-names(myres[2:ncol(myres)])
            mydat$values<-unname(t(myres[i,2:ncol(myres)]))
            mydat<-as.data.frame(cbind(mydat$metrics,mydat$values))
            colnames(mydat)<-c("metric","value")
            cat(paste("<b>",as.character(myres[i,1]) ,"</b><br>"))

            tbl <- kbl(mydat,format='pipe',digits=5,caption=as.character(t(myres[i,1]))) %>%
                    kable_styling("hover", full_width = FALSE)

            cat(tbl)
            cat('\n\n<!-- -->\n\n')
            cat("<br>")
            ndplot(i)
            cat("<br>")
            topgenelist(i)
            cat("<hr><br>")
        }
    }
}

```

## Network diagram

Only used for one-dimensional analysis.

Here, the network diagram is used to depict the similarity between some of the top ranked gene sets.
It makes separate charts for up and downregulated sets.
It works best when prioritisation is done by effect size during the `mitch_calc()` step.
By default, we only show the top 20 genes, but you can use the `networkplot()` command yourself
with other options.
See `?networkplot` for more detail.
There is an element of stochasticity with regard to the network projection, so if you see a lot of
overlapping labels or labels getting cut off, you could repeat the chart generation until you get a
nice layout.
See `?networkplot` for more detail.

Below the network diagrams, you will see lists of genes that make up the up and downregulated sets
respectively.
For upregulated genes the score needs to be >2 and for downregulated genes it needs to be < -2.
This is to remove genes that have uninteresting differential expression and do not contribute
enrichment.

```{r,network,fig.height=6, fig.width=14}

if (d==1) {
  networkplot(eres=res,FDR=0.05,n_sets=20)
  network_genes(eres=res,FDR=0.05,n_sets=20)
} else {
 message("Network charts only generated in one-dimensional analysis.")
}

```

## Session information

Here is the session info with all the versions of packages used.

```{r session_info, include=TRUE, echo=TRUE, results='markup'}

sessionInfo()

```

END of report

---
title: "Applying mitch to pathway analysis of Infinium Methylation array data"
author: "Antony Kaspi & Mark Ziemann"
date: "`r Sys.Date()`"
output:
  rmarkdown::html_vignette:
    toc: true
vignette: >
  %\VignetteIndexEntry{Applying mitch to pathway analysis of Infinium Methylation array data}
  %\VignetteEngine{knitr::rmarkdown}
  \usepackage[utf8]{inputenc}
---

## Background

Applying enrichment analysis to methylation array data is difficult due to the presence
of a variable number of probes per gene and the fact that a probe could belong to
overlapping genes.
There are existing over-representation based approaches to this, however they appear to
lack sensitivity.
To address this, we have developed a simple approach to aggregating differential
methylation data to make it suitable for downstream use by mitch.
The process begins with the differential probe methylation results from limma.
Here, we summarise the limma t-statistics by gene using the arithmetic mean.
The resulting gene level differential methylation scores then undergo mitch as usual.

## Requirements

In addition to mitch v1.15.0 of higher, you will need an annotation set for the array you are using.
These are conveniently provided as Bioconductor packages for HM450K and EPIC arrays.

HM450k: IlluminaHumanMethylation450kanno.ilmn12.hg19

EPIC: IlluminaHumanMethylationEPICanno.ilm10b4.hg19

One issue with these is that the annotations are quite old, which means that some of the gene
names have changed (~12%), so it is a good idea to screen the list of gene names and update
obsolete names using the HGNChelper package.

```{r, install, eval = FALSE}
if(!requireNamespace("BiocManager", quietly = TRUE))
    install.packages("BiocManager")
BiocManager::install("mitch")
```

```{r,lib}

suppressPackageStartupMessages({
    library("mitch")
    library("dplyr")
    library("HGNChelper")
    library("IlluminaHumanMethylation450kanno.ilmn12.hg19")
    library("IlluminaHumanMethylationEPICanno.ilm10b4.hg19")
})
```

## Gene sets

In this cut down example we will be using a sample of 200 Reactome gene sets.

```{r,genesetsExample}
data(genesetsExample)
head(genesetsExample,3)
```

## Probe gene relationship for HM450K array data

In order to get mitch working, we need a 2 column table that maps probes to genes.
The workflow shown here is for the HM450k array, and an EPIC example is show at the end of the report.

```{r,anno1}
anno <- getAnnotation(IlluminaHumanMethylation450kanno.ilmn12.hg19)
myann <- data.frame(anno[,c("UCSC_RefGene_Name","UCSC_RefGene_Group","Islands_Name","Relation_to_Island")])
head(myann)
gp <- myann[,"UCSC_RefGene_Name",drop=FALSE]
gp2 <- strsplit(gp$UCSC_RefGene_Name,";")
names(gp2) <- rownames(gp)
gp2 <- lapply(gp2,unique)
gt1 <- stack(gp2)
colnames(gt1) <- c("gene","probe")
gt1$probe <- as.character(gt1$probe)
dim(gt1)
head(gt1)
```

## Update deprecated gene symbols

Update old gene symbols using HGNChelper (13% of 21k names).
Last updated 4th Sep 2024.

```{r,genenames}
#new.hgnc.table <- getCurrentHumanMap()
new.hgnc.table <- readRDS("../inst/extdata/new.hgnc.table.rds")
fix <- checkGeneSymbols(gt1$gene,map=new.hgnc.table)
fix2 <- fix[which(fix$x != fix$Suggested.Symbol),]
length(unique(fix2$x))
gt1$gene <- fix$Suggested.Symbol
head(gt1)
```

## Importing profiling data
Here we will read in a table of differential probe methylation data generated by limma.
We will use the t-statistics for downstream analysis.

```{r,import1}
x <- read.table("https://ziemann-lab.net/public/gmea/dma3a.tsv",header=TRUE,row.names=1)
head(x)
```

Now that the profiling data is loaded, we need to import with mitch package, which
establishes the probe-gene relationships and aggregates the data to gene level scores.
As you can see, the input was a table of 422k probes and the output is 19,380 gene scores.
Many probes not annotated to genes are discarded.

```{r,import2}
y <- mitch_import(x,DEtype="limma",geneTable=gt1)
head(y)
dim(y)
```

## Calculating enrichment

The `mitch_calc` function performs an enrichment test.
If you imported multiple data tables in the previous step, mitch will conduct a
multivariate enrichment test.
The results can be prioritised by significance or effect size.
My recommendation is to discard results with FDR>0.05 then prioritise by effect
size, which for us is the mitch enrichment score called S distance.
In this example I also set the minimum gene set size to 5.

```{r,enrich1}
res<-mitch_calc(y,genesetsExample,priority="effect",cores=2,minsetsize=5)
head(res$enrichment_result,10)
```

## Downstream presentation

For presentation of the results you could consider making a volcano plot and/or
making a barplot of S.dist values for selected gene sets that meet the FDR cutoff.
You can also use the built in functions to make a set of charts and html report.
It is vitally important to inspect the detailed resports to understand which
genes are driving the enrichment in your dataset.

```{r,downstream1}

mitch_plots(res,outfile="methcharts.pdf")

```

```{r,downstream2,results=FALSE}

mitch_report(res,"methreport.html")

```

We have a `networkplot()` function that produces two network plots, one for increased
methylation (nodes shaded red), and one for decreased methylation (nodes shaded blue).
See `?networkplot()` for more details.

Lists of enriched gene set members are shown.

```{r,meth_network,fig.width=14,fig.height=6}

networkplot(res,FDR=0.1,n_sets=10)

network_genes(res,FDR=1,n_sets=10)

```

## Probe gene relationship for EPIC array data

Here is the code to create a probe-gene table for EPIC array data.
Be sure to update the gene symbols with `HGNChelper` before conducting enrichment analysis.

```{r,EPIC}
anno <- getAnnotation(IlluminaHumanMethylationEPICanno.ilm10b4.hg19)
myann <- data.frame(anno[,c("UCSC_RefGene_Name","UCSC_RefGene_Group","Islands_Name","Relation_to_Island")])
gp <- myann[,"UCSC_RefGene_Name",drop=FALSE]
gp2 <- strsplit(gp$UCSC_RefGene_Name,";")
names(gp2) <- rownames(gp)
gp2 <- lapply(gp2,unique)
gt <- stack(gp2)
colnames(gt) <- c("gene","probe")
gt$probe <- as.character(gt$probe)
dim(gt)
str(gt)
```

## Session Info
```{r,sessioninfo,message=FALSE}
sessionInfo()
```

---
title: "mitch Workflow"
author: "Antony Kaspi & Mark Ziemann"
date: "`r Sys.Date()`"
output:
  rmarkdown::html_vignette:
    toc: true
vignette: >
  %\VignetteIndexEntry{mitch Workflow}
  %\VignetteEngine{knitr::rmarkdown}
  \usepackage[utf8]{inputenc}
---

## Background

mitch is an R package for multi-contrast enrichment analysis. At
it’s heart, it uses a rank-MANOVA based statistical approach to detect sets
of genes that exhibit enrichment in the multidimensional space as compared
to the background. The rank-MANOVA concept dates to work by Cox and Mann
(https://doi.org/10.1186/1471-2105-13-S16-S12). mitch is useful for
pathway analysis of profiling studies with one, two or more contrasts,
or in studies with multiple omics profiling, for example proteomic,
transcriptomic, epigenomic analysis of the same samples. mitch is perfectly
suited for pathway level differential analysis of scRNA-seq data.

The main strengths of mitch are that it can import datasets easily
from many upstream tools and has advanced plotting features to
visualise these enrichments. mitch consists of five functions. A
typical mitch workflow would consist of:

* 1: Import gene sets with `gmt_import()`

* 2: Import profiling data with `mitch_import()`

* 3: Calculate enrichments with `mitch_calc()`

* 4: And generate plots and reports with `mitch_plots()`, `mitch_report()` and
`networkplot()`.

```{r, install, eval = FALSE}
if(!requireNamespace("BiocManager", quietly = TRUE))
    install.packages("BiocManager")
BiocManager::install("mitch")
```

```{r, lib}
library("mitch")
```

## Importing gene sets
mitch has a function to import GMT files to R lists (adapted from Yu et al, 2012
in the [clusterProfiler](https://bioconductor.org/packages/release/bioc/html/clusterProfiler.html) 
package). For example we can grab some gene sets from Reactome.org:

```{r,gsets}
download.file("https://reactome.org/download/current/ReactomePathways.gmt.zip",
    destfile="ReactomePathways.gmt.zip")
unzip("ReactomePathways.gmt.zip")
genesets <- gmt_import("ReactomePathways.gmt")
```

In this cut down example we will be using a sample of 200 Reactome gene sets:

```{r,genesetsExample}
data(genesetsExample)
head(genesetsExample,3)
```

## Importing profiling data
mitch accepts pre-ranked data supplied by the user, but also has a function
called `mitch_import` for importing tables generated by limma, edgeR, DESeq2,
ABSSeq, Sleuth, Seurat, Muscat and several other upstream tools. By default,
only the genes that are detected in all contrasts are included, but this 
behaviour can be modified for sparse data setting `joinType=full`. The below
example imports two edgeR tables called "rna" and "k9a" Where gene 
identifiers are present as row names. Note that if there is more than one
profile being imported, they need to be part of a list.

```{r,import11}
data(rna,k9a)
x <- list("rna"=rna,"k9a"=k9a)
y <- mitch_import(x,"edgeR")
head(y)
```

mitch can do unidimensional analysis if you provide it a single profile as a
dataframe (not in a list).

```{r,import4}
y <- mitch_import(rna,DEtype="edger")
head(y)
```

If the gene identifiers are not given in the rownames, then the column can be
specified with the `geneIDcol` parameter like this:

```{r,import5}
# first rearrange cols
rna_mod <- rna
rna_mod$MyGeneIDs <- rownames(rna_mod)
rownames(rna_mod) <- seq(nrow(rna_mod))
head(rna_mod)
# now import with geneIDcol
y <- mitch_import(rna_mod,DEtype="edgeR",geneIDcol="MyGeneIDs")
head(y)
```

By default, differential gene activity is scored using a supplied test
statistic or directional p-value (D):

D = sgn(logFC) * -log10(p-value)

If this is not desired, then users can perform their own custom scoring
procedure and import with `DEtype="prescored"`.

There are many cases where the gene IDs don't match the gene sets. To overcome
this, `mitch_import` also accepts a two-column table (gt here) that relates gene
identifiers in the profiling data to those in the gene sets. In this example
we can create some fake gene accession numbers to demonstrate this feature.

```{r,import6}
library("stringi")
# obtain vector of gene names
genenames <- rownames(rna)
# create fake accession numbers
accessions <- paste("Gene0",stri_rand_strings(nrow(rna)*2, 6, pattern = "[0-9]"),sep="")
accessions <- head(unique(accessions),nrow(rna))
# create a gene table file that relates gene names to accession numbers
gt <- data.frame(genenames,accessions)

# now swap gene names for accessions
rna2 <- merge(rna,gt,by.x=0,by.y="genenames")
rownames(rna2) <- rna2$accessions
rna2 <- rna2[,2:5]

k9a2 <- merge(k9a,gt,by.x=0,by.y="genenames")
rownames(k9a2) <- k9a2$accessions
k9a2 <- k9a2[,2:5]

# now have a peek at the input data before importing
head(rna2,3)
head(k9a2,3)
head(gt,3)
x <- list("rna2"=rna2,"k9a2"=k9a2)
y <- mitch_import(x,DEtype="edgeR",geneTable=gt)
head(y,3)
```

`?mitch_import` provides more instructions on using this feature.

## Calculating enrichment
The `mitch_calc` function performs multivariate enrichment analysis of the
supplied gene sets in the scored profiling data. At its simpest form
`mitch_calc` function accepts the scored data as the first argument and the
genesets as the second argument. Users can prioritise enrichments based on
small adjusted p-values, by the observed effect size ("s.dist", the magnitude
of "s", the enrichment score) or the standard deviation of the s scores.
For most analyses, you will see more specific results when prioritising by
effect, but it is worth looking at both.

Note that the number of parallel threads ("cores") is set here to cores=2 but
the default is to use just one.
If you have many available threads, then a good number is 8-12.
Note that the more CPU threads used, the more RAM is required, so monitor the
available memory on your system.

```{r,calc1,results="hide"}
# prioritisation by significance
res <- mitch_calc(y,genesetsExample,priority="significance",cores=2)
```

```{r,calc2}
# peek at the results
head(res$enrichment_result)
```

```{r,calc3,results="hide"}
# prioritisation by effect size
res <- mitch_calc(y,genesetsExample,priority="effect",cores=2)
```

```{r,calc4}
head(res$enrichment_result)
```

By default, gene sets with fewer than 10 members present in the profiling data
are discarded. This threshold can be modified using the minsetsize option.
There is no upper limit of gene set size.

```{r,calc5,results="hide"}
res <- mitch_calc(y,genesetsExample,priority="significance",minsetsize=5,cores=2)
```

By default, in downstream visualisation steps, charts are made from the top 50 gene
sets, but this can be modified using the resrows option.

```{r,calc6,results="hide"}
res<-mitch_calc(y,genesetsExample,priority="significance",resrows=3,cores=2)
```

## Generate a HTML report
The HTML reports contain several plots as raster images and interactive charts
which are useful as a first-pass visualisation. These can be generated like
this:

```{r,report,results="hide"}
mitch_report(res,"myreport.html",overwrite=TRUE)
```

## Generate high resolution plots
In case you want the charts in PDF format, for example for publications, these
can be generated as such:

```{r,plots,results="hide"}
mitch_plots(res,outfile="mycharts.pdf")
```

## Network plot

For one-dimensional analysis, you can generate a network plot to visualise the
similarity of the driver gene members for the top (20) up- and downregulated sets
meeting a significance cut-off (FDR<0.05).
These cut-offs can be altrered.

By default, this includes genes scored in the top and bottom tertiles, otherwise
they are discarded.
The intensity of the colour is proportional to the s.dist value (enrichment score).
Circle size is proportional to the number of genes in the set.
Line thickness is proportional to the Jaccard similarity value.
This function works best after prioritisation with "effect" when running
`mitch_calc()`.
Note that the circle (node) size and the line width shown in the legend is approximate,
although the values shown are exactly the smallest and largest respectively.
Note that this chart works best when the width is double the height, otherwise
many of the long gene set names could be cut off.
There is an element of stochasticity with regard to the network projection, so
it could be a good idea to repeat it a few times until you get a nice layout.

Lists of enriched gene set members are shown.

```{r,network_demo,fig.width=14,fig.height=6}

networkplot(res,FDR=1,n_sets=10)

network_genes(res,FDR=1,n_sets=10)

```

## Session Info

```{r,sessioninfo,message=FALSE}
sessionInfo()
```

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mitch.R
\name{mitch_plots}
\alias{mitch_plots}
\title{mitch_plots}
\usage{
mitch_plots(res, outfile = "Rplots.pdf")
}
\arguments{
\item{res}{a mitch results object.}

\item{outfile}{the destination file for the plots in PDF format. should
contain 'pdf' suffix. Defaults to 
'Rplots.pdf'}
}
\value{
generates a PDF file containing enrichment plots.
}
\description{
This function generates several plots of multivariate gene set enrichment in
high resolution PDF format.
The number of detailed sets to generate is dictated by the resrows set in
the mitch_calc command.
}
\examples{
data(resExample)
mitch_plots(resExample,outfile='outres.pdf')
}
\keyword{mitch}
\keyword{pdf}
\keyword{plot}
\keyword{plots}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mitch.R
\docType{data}
\name{rna}
\alias{rna}
\title{RNA profile}
\format{
data frame
}
\usage{
data(rna)
}
\description{
Example edgeR result of differential RNA expression.
This is a dataframe which contains columns for log fold change, log counts
per million, p-value and FDR adjusted p-value. These columns consist of 
numerical values. The row names represent human gene names. This is a sample
of 1000 gene of an original dataset that contains measurements of ~15000 
genes.
}
\examples{
data(rna)
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mitch.R
\docType{data}
\name{genesetsExample}
\alias{genesetsExample}
\title{Reactome gene sets}
\format{
A list of gene sets
}
\source{
Reactome website: https://reactome.org/
}
\usage{
data(genesetsExample)
}
\description{
Genesets from Reactome database suitable for enrichment analysis.
Acquired August 2019. The structure of this data is a named list of
vectors, containing human gene names (character strings). This is 
a sample of 200 gene sets from the approximately 2000 present in the
full dataset.
}
\examples{
data(genesetsExample)
}
\references{
Fabregat et al. (2017) BMC Bioinformatics volume 18, 
Article number: 142, https://www.ncbi.nlm.nih.gov/pubmed/28249561
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mitch.R
\name{network_genes}
\alias{network_genes}
\title{network_genes}
\usage{
network_genes(eres, FDR = 0.05, n_sets = 20)
}
\arguments{
\item{eres}{a mitch results object.}

\item{FDR}{the significance threshold for inclusion. By default, it is 0.05}

\item{n_sets}{the number of sets to include.}
}
\value{
a list of up- and down-regulated gene sets
}
\description{
This is a companion function for the network diagrams to display the genes which contribute
to the enrichment and are shared between gene sets.
}
\examples{
data(resExample)
network_genes(resExample)
}
\keyword{mitch}
\keyword{network}
\keyword{visualisation}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mitch.R
\docType{data}
\name{myImportedData}
\alias{myImportedData}
\title{myImportedData: Example imported profiles}
\format{
data frame
}
\usage{
data(myImportedData)
}
\description{
Example of three edgeR profiles imported using mitch.
The structure of this data is a dataframe where each column represents
one of the following profiling datasets after scoring: RNA, H3K9ac and
H3K36ac. Each row represents one gene and this dataset contains just 1000
rows to keep the example dataset small.
}
\examples{
data(myImportedData)
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mitch.R
\name{gmt_import}
\alias{gmt_import}
\title{gmt_import}
\usage{
gmt_import(gmtfile)
}
\arguments{
\item{gmtfile}{a gmt file.}
}
\value{
a list of gene sets.
}
\description{
This function imports GMT files into a list of character vectors for mitch 
analysis. GMT files are a commonly used
format for lists of genes used in pathway enrichment analysis. GMT files can
be obtained from Reactome, MSigDB, etc.
}
\examples{
# Import some gene sets
genesetsExample<-gmt_import(system.file('extdata/sample_genesets.gmt', 
package = 'mitch'))
}
\keyword{genesets}
\keyword{import}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mitch.R
\docType{package}
\name{mitch}
\alias{mitch-package}
\alias{mitch}
\title{mitch: An R package for multi-dimensional pathway enrichment analysis}
\description{
mitch is an R package for multi-dimensional enrichment analysis. At it's 
heart, it uses a rank-MANOVA based statistical approach to detect sets of 
genes that exhibit enrichment in the multidimensional space as compared to 
the background. mitch is useful for pathway analysis of profiling studies 
with two to or more contrasts, or in studies with multiple omics profiling, 
for example proteomic, transcriptomic, epigenomic analysis of the same 
samples. mitch is perfectly suited for pathway level differential analysis 
of scRNA-seq data.
}
\details{
A typical mitch workflow consists of:
1) Import gene sets with gmt_import()
2) Import profiling data with mitch_import()
3) Calculate enrichments with mitch_calc()
4) And generate plots and reports with mitch_plots() and mitch_report()

More documentation on the github page https://github.com/markziemann/mitch
or with ?<function>, eg: ?mitch_import
}
\examples{
# Example workflow
# Import some gene sets
genesetsExample<-gmt_import(system.file('extdata/sample_genesets.gmt', 
package = 'mitch'))
# Load some edgeR tables (rna, k9a, k36a). 
data(rna,k9a,k36a)
# Create a list of differential profiles
myList<-list('rna'=rna,'k9a'=k9a,'k36a'=k36a)
# Import as edgeR table 
myImportedData<-mitch_import(myList,DEtype='edger')
# Calculate enrichment using MANOVA
resExample<-mitch_calc(myImportedData,genesetsExample,priority='effect',
resrows=5,cores=2)
# Generate some high res plots in PDF format
mitch_plots(resExample,outfile='outres.pdf')
#' Generate a report of the analysis in HTML format
mitch_report(resExample,'outres.html')
}
\seealso{
Useful links:
\itemize{
  \item \url{https://github.com/markziemann/mitch}
  \item Report bugs at \url{https://github.com/markziemann/mitch}
}

}
\author{
\strong{Maintainer}: Mark Ziemann \email{mark.ziemann@gmail.com} (\href{https://orcid.org/0000-0002-7688-6974}{ORCID}) [copyright holder]

Authors:
\itemize{
  \item Antony Kaspi [copyright holder]
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mitch.R
\name{networkplot}
\alias{networkplot}
\title{networkplot}
\usage{
networkplot(eres, FDR = 0.05, n_sets = 20)
}
\arguments{
\item{eres}{a mitch results object.}

\item{FDR}{the significance threshold for inclusion. By default, it is 0.05}

\item{n_sets}{the number of sets to include. This type of graph can get messy,
so the default is 20, but it can be customised to your needs.}
}
\value{
generates network diagrams.
}
\description{
This function generates gene set network diagrams to complement the enrichment analysis.
It produces separate charts for up and down-regulated sets.
This only includes genes ranked in the top and bottom tertiles.
Genes in the set which do not meet this criterion are discarded.
The intensity of the colour is proportional to the s.dist value (enrichment score).
Circle size is proportional to the number of genes in the set.
Line thickness is proportional to the Jaccard similarity value.
This function works best after prioritisation with "effect" when running
mitch_calc().
Note that the circle size and the line width shown in the legend is approximate,
although the values shown are exactly the smallest and largest respectively.
Note that this chart works best when the width is double the height, otherwise
many of the long gene set names could be cut off.
There is an element of stochasticity with regard to the network projection, so
it could be a good idea to repeat it a few times until you get a nice layout.
}
\examples{
data(resExample)
networkplot(resExample)
}
\keyword{mitch}
\keyword{network}
\keyword{visualisation}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mitch.R
\docType{data}
\name{myList}
\alias{myList}
\title{myList: A list of three edgeR results}
\format{
data frame
}
\usage{
data(myList)
}
\description{
Example edgeR results of differential RNA, H3K9ac and H3K36ac profiling.
The structure of this data is a list of three dataframes. Each data frame
is 1000 lines only.
}
\examples{
data(myList)
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mitch.R
\docType{data}
\name{k9a}
\alias{k9a}
\title{H3K9ac profile}
\format{
data frame
}
\usage{
data(k9a)
}
\description{
Example edgeR result of differential ChIP-seq H3K9ac.
This is a dataframe which contains columns for log fold change, log counts
per million, p-value and FDR adjusted p-value. These columns consist of 
numerical values. The row names represent human gene names. This is a sample
of 1000 gene of an original dataset that contains measurements of ~30000 
genes.
}
\examples{
data(k9a)
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mitch.R
\name{mitch_calc}
\alias{mitch_calc}
\title{mitch_calc}
\usage{
mitch_calc(
  x,
  genesets,
  minsetsize = 10,
  cores = 1,
  resrows = 50,
  priority = NULL
)
}
\arguments{
\item{x}{a multicolumn numerical table with each column containing 
differential expression scores for a contrast.
Rownames must match genesets.}

\item{genesets}{lists of genes imported by the gmt_imprt function or
similar.}

\item{minsetsize}{the minimum number of genes required in a set for it to be
included in the statistical analysis.
Default is 10.}

\item{cores}{the number of parallel threads for computation. Defaults to 1.}

\item{resrows}{an integer value representing the number of top genesets for
which a detailed report is to be 
generated. Default is 50.}

\item{priority}{the prioritisation metric used to selecting top gene sets. 
Valid options are 'significance', 
'effect' and 'SD'.}
}
\value{
mitch res object with the following parts:
$input_profile: the supplied input differential profile
$input_genesets: the supplied input gene sets
$ranked_profile: the differential profile after ranking
$enrichment_result: the table of MANOVA/ANOVA enrichment results for each
gene set
$analysis_metrics:  several metrics that are important to the interpretation
of the results
$detailed_sets: a list of dataframes containing ranks of members of
prioritised gene sets.
}
\description{
This function performs multivariate gene set enrichment analysis.
}
\examples{
# Example using mitch to calculate multivariate enrichments and
# prioritise based on effect size 
data(myImportedData,genesetsExample)
resExample<-mitch_calc(myImportedData,genesetsExample,priority='effect',
minsetsize=5,cores=2)
}
\keyword{calc}
\keyword{calculate}
\keyword{manova}
\keyword{mitch}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mitch.R
\name{mitch_import}
\alias{mitch_import}
\title{mitch_import}
\usage{
mitch_import(x, DEtype, geneIDcol = NULL, geneTable = NULL, joinType = NULL)
}
\arguments{
\item{x}{a list of differential expression tables.}

\item{DEtype}{the program that generated the differential expression table
Valid options are 'edgeR', 'DESeq2', 'limma', 'ABSSeq', 'Sleuth', 'Seurat',
'topConfects', 'muscat', 'Swish', 'scDE', 'MAST', 'DEsingle', 'ballgown',
'NOIseq', 'TCC', 'DEDS', 'cuffdiff', 'fishpond', 'missMethyl', 'DMRcate',
'DEP', 'msmsTests', 'plgem', 'SDAMS', 'DEqMS', 'DiffBind' and 'prescored'. 
Where 'prescored' is a dataframe containing the test statistic and gene ID 
(either in rowname or separate column) and nothing else. 'preranked' is an 
alias for 'prescored'.}

\item{geneIDcol}{the column containing gene names. If gene names are 
specified as row names, then geneIDcol=NULL.}

\item{geneTable}{a 2 column table mapping gene identifiers in the profile to
gene identifiers in the gene sets.}

\item{joinType}{the type of join to perform, either 'inner' or 'full'.
By default, joins are 'inner' except for Seurat and muscat where full is 
used.}
}
\value{
a multi-column table compatible with mitch_calc analysis.
}
\description{
This function imports differential omics data from common differential tools
like edgeR, limma and DESeq2. It calculates a summarised differential
expression metric by multiplying the sign of the log fold change by the 
-log10 of the p-value. If this behaviour is not desired, mitch_import can be
bypassed in favour of another scoring metric.
}
\examples{
# first step is to create a list of differential profiles
data(rna,k9a,k36a)
x<-list('rna'=rna,'k9a'=k9a,'k36a'=k36a)
# import as edgeR table 
imported<-mitch_import(x,DEtype='edger')
}
\keyword{import}
\keyword{mitch}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mitch.R
\docType{data}
\name{resExample}
\alias{resExample}
\title{resExample: Example mitch result}
\format{
list of mixed data types
}
\usage{
data(resExample)
}
\description{
Example of mitch results. Enrichment of the Reactome gene sets in the RNA,
H3K9ac and H3K36ac datasets. The structure of this data set is a list where
the 1st element is "input_profile" that has been imported (data frame), 2nd
element is the "input_genesets" (names list of gene names [character 
vectors]), 3rd is "ranked_profile" which is the input profiling data after
ranking (data frame), 4th is "enrichment_result" which is a data frame which
provides enrichment information on each gene set in the profiling data
including s scores, p-values and FDR adjusted p-values. 5th is 
"analysis_metrics" (list). The 6th slot is "detailed_sets" which is a list
of 5 matrices which details the enrichment of members of selected gene sets.
}
\examples{
data(resExample)
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mitch.R
\name{mitch_report}
\alias{mitch_report}
\title{mitch_report}
\usage{
mitch_report(res, outfile, overwrite = FALSE)
}
\arguments{
\item{res}{a mitch results object.}

\item{outfile}{the destination file for the html report. should contain
'html' suffix. Defaults to 
'Rplots.pdf'}

\item{overwrite}{should overwrite the report file if it already exists?}
}
\value{
generates a HTML file containing enrichment plots.
}
\description{
This function generates an R markdown based html report containing tables
and several plots of mitch results 
The plots are in png format, so are not as high in resolution as compared to
the PDF generated by mitch_plots 
function. The number of detailed sets to generate is dictated by the resrows
set in the mitch_calc command.
}
\examples{
data(resExample)
mitch_report(resExample,'outres2.html')
}
\keyword{html}
\keyword{knitr}
\keyword{markdown}
\keyword{mitch}
\keyword{report}
